@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,5 +24,5 @@ class PngDC : public GTLImageIO::ImageDC {
     PngDC();
     virtual ~PngDC();
     virtual GTLCore::AbstractImage* decode( const GTLCore::String& _fileName, GTLCore::Region*  = 0, GTLCore::String* _errorMessage = 0 ) const;
-    virtual bool encode( const GTLCore::AbstractImage* _image, const GTLCore::Region& _region, const GTLCore::String& _fileName, GTLCore::String* _errorMessage = 0 ) const;
+    virtual bool encode( const GTLCore::AbstractImage* _image, const GTLCore::Region& _region, const GTLCore::String& _fileName, const GTLImageIO::Options* _options = 0, GTLCore::String* _errorMessage = 0 ) const;
 };
