@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,12 +33,14 @@
 #include "GTLCore/Type_p.h"
 #include "GTLCore/Value.h"
 #include "GTLCore/VariableNG_p.h"
+#include "GTLCore/BufferImage.h"
 
 // OpenCTL
 #include "Module.h"
 #include "GTLCore/ModuleData_p.h"
 #include "Debug.h"
 #include "GTLCore/VirtualMachine_p.h"
+#include "GTLCore/Optimiser_p.h"
 
 // LLVM
 #include <llvm/DerivedTypes.h>
@@ -50,7 +52,6 @@
 // Passes
 #include <llvm/PassManager.h>
 #include <llvm/Analysis/LoopPass.h>
-#include <llvm/Analysis/LoadValueNumbering.h>
 #include <llvm/Analysis/Verifier.h>
 #include <llvm/Target/TargetData.h>
 #include <llvm/ExecutionEngine/ExecutionEngine.h>
@@ -116,7 +117,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module, co
   // Clone the module
   if( module->data())
   {
-    d->module = llvm::CloneModule(module->data()->llvmModule());
+    d->module = llvm::CloneModule(module->data()->llvmLinkedModule());
     d->moduleData = new GTLCore::ModuleData( d->module );
     const GTLCore::Function* functionDef = module->function( functionName );
     if( not functionDef ) return;
@@ -190,9 +191,9 @@ void Program::init(const GTLCore::String& functionName, const Module* module, co
           GTLCore::VariableNG* vng = 0;
           if(dstChannelsTypes[i] == GTLCore::Type::Half)
           {
-            vng = new GTLCore::VariableNG(GTLCore::Type::Float, false );
+            vng = new GTLCore::VariableNG(GTLCore::Type::Float, false, false );
           } else {
-            vng = new GTLCore::VariableNG(GTLCore::Type::Integer32, false );
+            vng = new GTLCore::VariableNG(GTLCore::Type::Integer32, false, false );
           }
           vng->initialise( gc, initialBlock, GTLCore::ExpressionResult(), std::list<llvm::Value*>());
           buffer[i] = vng;
@@ -214,9 +215,9 @@ void Program::init(const GTLCore::String& functionName, const Module* module, co
     // Construct the "conditions" of the loop
       //   int i = 0;
       OCTL_DEBUG("int i = 0");
-      GTLCore::VariableNG* posSrc = new GTLCore::VariableNG( GTLCore::Type::Integer32, false);
+      GTLCore::VariableNG* posSrc = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false );
       posSrc->initialise( gc, initialBlock, GTLCore::ExpressionResult(cg.integerToConstant(0), GTLCore::Type::Integer32), std::list<llvm::Value*>());
-      GTLCore::VariableNG* posDst = new GTLCore::VariableNG( GTLCore::Type::Integer32, false);
+      GTLCore::VariableNG* posDst = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false );
       posDst->initialise( gc, initialBlock, GTLCore::ExpressionResult(cg.integerToConstant(0), GTLCore::Type::Integer32), std::list<llvm::Value*>());
       // i < size
       OCTL_DEBUG("i < size");
@@ -272,7 +273,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module, co
         {
           OCTL_ASSERT(buffer[i]);
           buffer[i]->set( gc, bodyBlock, new llvm::LoadInst(pointer,"", bodyBlock), dstPixelDescription.channelTypes()[i]);
-          arguments.push_back( buffer[i]->pointer());
+          arguments.push_back( buffer[i]->pointer(bodyBlock));
         } else {
           arguments.push_back(pointer);
         }
@@ -306,7 +307,7 @@ void Program::init(const GTLCore::String& functionName, const Module* module, co
       OCTL_DEBUG( *function->getFunctionType());
 #endif
       llvm::CallInst *CallFunc = llvm::CallInst::Create(function, arguments.begin(), arguments.end(), "", bodyBlock);
-      CallFunc->setTailCall();
+      CallFunc->setTailCall(false);
       // If there was buffering, save to output
       OCTL_DEBUG("If there was buffering, save to output");
       for(int i = 0; i < dstCountChannels; ++i)
@@ -337,15 +338,10 @@ void Program::init(const GTLCore::String& functionName, const Module* module, co
       OCTL_DEBUG("return;");
       llvm::ReturnInst::Create(finBlock);
       OCTL_DEBUG(*d->module);
-      // Optimize, FIXME: use GTLCore's optimizer
+      // Optimize
       OCTL_DEBUG("Optimize");
-      llvm::PassManager Passes;
-      // Add in the passes we want to execute
-      Passes.add(new llvm::TargetData(d->module));
-      // Verify we start with valid
-      Passes.add(llvm::createVerifierPass());
-      // Run
-      Passes.run(*d->module);
+      GTLCore::Optimiser::instance()->d->passManager()->run( *d->module );
+      
       // Register module in the VM
       d->moduleProvider = new llvm::ExistingModuleProvider( d->module );
       GTLCore::VirtualMachine::instance()->registerModule( d->moduleProvider );
@@ -406,6 +402,21 @@ void Program::apply(const GTLCore::Buffer& input, GTLCore::Buffer& output) const
   OCTL_DEBUG("Applied");
 }
 
+void Program::apply(const GTLCore::AbstractImage& input, GTLCore::AbstractImage& output) const
+{
+  GTLCore::AbstractImage::ConstIterator* itSrc = input.createIterator();
+  GTLCore::AbstractImage::Iterator* itDst = output.createIterator();
+  
+  while( itSrc->next() and itDst->next())
+  {
+    apply( *itSrc->current(), *itDst->current() );
+  }
+  
+  delete itDst;
+  delete itSrc;
+}
+
+
 void Program::setVarying( const GTLCore::String& _name, const GTLCore::Value& _value )
 {
   std::map< GTLCore::String, void*>::iterator it = d->varyingsPtr.find( _name );
