@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,19 +20,25 @@
 #ifndef _OPENCTL_OPENCTL_H_
 #define _OPENCTL_OPENCTL_H_
 
+#include <GTLCore/Version.h>
+
+#define OPENCTL_VERSION_MAJOR GTL_CORE_VERSION_MAJOR
+#define OPENCTL_VERSION_MINOR GTL_CORE_VERSION_MINOR
+#define OPENCTL_VERSION_REVISION GTL_CORE_VERSION_REVISION
+
 namespace OpenCTL {
-  const char* LibraryShortName = "OpenCTL";
-  const char* LibraryName = "Open Color Transformation Language";
-  const char* LibraryCopyright = "Copyright (c) 2007-2008 Cyrille Berger (cberger@cberger.net)";
-  const char* LibraryLicence = "GNU Lesser General Public License Version 2";
-  const char* LibraryVersionString = "0.9.5";
-  int LibraryVersionMajor = 0;
-  int LibraryVersionMinor = 9;
-  int LibraryVersionRevision = 5;
+  GTLCore::String LibraryShortName();
+  GTLCore::String LibraryName();
+  GTLCore::String LibraryCopyright();
+  GTLCore::String LibraryLicence();
+  GTLCore::String LibraryVersionString();;
+  int LibraryVersionMajor();
+  int LibraryVersionMinor();
+  int LibraryVersionRevision();
   /**
    * Give the latest version of the CTL language supported by this library.
    */
-  int LanguageVersion = 1;
+  int LanguageVersion();
 }
 
 #endif
