@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,6 +19,7 @@
 
 #include "CodeGenerator_p.h"
 
+#include <llvm/CallingConv.h>
 #include <llvm/Constants.h>
 #include <llvm/Function.h>
 #include <llvm/Instructions.h>
@@ -35,8 +36,11 @@
 #include "Value.h"
 #include "VariableNG_p.h"
 #include "ModuleData_p.h"
+#include "TypesManager_p.h"
 
 #include "AST/AccessorExpression.h"
+#include "wrappers/ArrayWrap.h"
+#include "MemoryModelConfig_p.h"
 
 #define UNIFORMIZE_TYPES( _v1_, _v2_) \
   GTL_ASSERT( _v1_.value() ); \
@@ -118,6 +122,47 @@ llvm::Constant* CodeGenerator::valueToConstant( const GTLCore::Value& v)
   return 0;
 }
 
+llvm::Value* CodeGenerator::createVector( const GTLCore::Type* _vecType)
+{
+  llvm::Constant* _const = 0;
+  switch( _vecType->embeddedType()->dataType() )
+  {
+    case Type::UNSIGNED_INTEGER32:
+    case Type::INTEGER32:
+      _const = integerToConstant( 0 );
+      break;
+    case Type::FLOAT:
+    case Type::HALF:
+      _const = floatToConstant( 0.0 );
+      break;
+    default:  
+      break;
+  }
+  GTL_ASSERT( _const );
+  return createVector( _vecType, _const );
+}
+
+llvm::Value* CodeGenerator::createVector( const GTLCore::Type* _vecType, llvm::Constant* _val)
+{
+  GTL_ASSERT( _vecType->dataType() == Type::VECTOR );
+  std::vector<llvm::Constant*> inits;
+  for( std::size_t i = 0; i < _vecType->vectorSize(); ++i)
+  {
+    inits.push_back( _val );
+  }
+  return llvm::ConstantVector::get( static_cast<const llvm::VectorType*>( _vecType->d->type()), inits );
+}
+
+llvm::Value* CodeGenerator::createVector( int _size, llvm::Constant* _val, const GTLCore::Type* _valType)
+{
+  std::vector<llvm::Constant*> inits;
+  for( int i = 0; i < _size; ++i)
+  {
+    inits.push_back( _val );
+  }
+  return llvm::ConstantVector::get( static_cast<const llvm::VectorType*>( GTLCore::TypesManager::getVector( _valType, _size)->d->type()), inits );
+}
+
 llvm::Value* CodeGenerator::convertPointerToCharP(llvm::BasicBlock* _currentBlock, llvm::Value* _value)
 {
   return convertPointerTo( _currentBlock, _value, llvm::IntegerType::get(8) );
@@ -147,10 +192,10 @@ llvm::Value* CodeGenerator::convertValueTo(llvm::BasicBlock* _currentBlock, llvm
   {
     GTL_DEBUG("Convert value to a vector");
     // Create a vector
-    llvm::Value* resultLoad = llvm::ConstantVector::get( static_cast<const llvm::VectorType*>(_targetType->d->type()), std::vector<llvm::Constant*>() );
+    llvm::Value* resultLoad = createVector( _targetType, floatToConstant( 0.0 ) );
     llvm::Value* vecElt = convertValueTo( _currentBlock, _value, _valueType, _targetType->embeddedType() );
     // Affect the same value to each element of the vector
-    for(int i = 0; i < _targetType->vectorSize(); ++i)
+    for(unsigned int i = 0; i < _targetType->vectorSize(); ++i)
     {
       resultLoad = llvm::InsertElementInst::Create( resultLoad, vecElt, i, "", _currentBlock );
     }
@@ -161,7 +206,9 @@ llvm::Value* CodeGenerator::convertValueTo(llvm::BasicBlock* _currentBlock, llvm
   GTL_DEBUG( *_value << " to "  << *_targetType );
   GTL_ASSERT(_targetType->d->type()->isFirstClassType());
   GTL_ASSERT(_value->getType()->isFirstClassType());
-  return llvm::CastInst::create(
+  GTL_ASSERT( _targetType->dataType() != Type::ARRAY );
+  GTL_ASSERT( _targetType->dataType() != Type::STRUCTURE );
+  return llvm::CastInst::Create(
                       llvm::CastInst::getCastOpcode(_value, _valueType->isSigned(), _targetType->d->type(), _targetType->isSigned()), _value, _targetType->d->type(), "", _currentBlock );
 }
 
@@ -177,7 +224,7 @@ llvm::Constant* CodeGenerator::convertConstantTo(llvm::Constant* _constant, cons
     llvm::Constant* vecElt = convertConstantTo( _constant, _constantType, _targetType->embeddedType() );
     // Affect the same value to each element of the vector
     std::vector<llvm::Constant*> constants;
-    for(int i = 0; i < _targetType->vectorSize(); ++i)
+    for(unsigned int i = 0; i < _targetType->vectorSize(); ++i)
     {
       constants.push_back( vecElt );
     }
@@ -289,7 +336,7 @@ void CodeGenerator::createWhileStatement( llvm::BasicBlock* before, llvm::BasicB
 
 llvm::Function* CodeGenerator::createFunction( llvm::FunctionType* type, const GTLCore::String& name)
 {
-  return llvm::Function::Create( type, llvm::GlobalValue::LinkOnceLinkage,
+  return llvm::Function::Create( type, llvm::GlobalValue::ExternalLinkage,
                   name, d->module->llvmModule() );
 }
 
@@ -334,7 +381,7 @@ void CodeGenerator::createDecrementExpression( llvm::BasicBlock* _currentBlock,
   llvm::Value* CodeGenerator::_GTL_Function_Name_(llvm::BasicBlock* currentBlock, llvm::Value* lhs, const Type* lhsType, llvm::Value* rhs, const Type* rhsType) \
   { \
     UNIFORMIZE_VALUE_TYPES( lhs, rhs ) \
-    return llvm::BinaryOperator::_LLVM_Binary_Operator( v1, v2, "", currentBlock ); \
+    return llvm::BinaryOperator::_LLVM_Binary_Operator( v1, v2, #_GTL_Function_Name_, currentBlock ); \
   } \
   llvm::Constant* CodeGenerator::_GTL_Function_Name_( llvm::Constant* lhs, const Type* lhsType, llvm::Constant* rhs, const Type* rhsType) \
   { \
@@ -350,11 +397,11 @@ void CodeGenerator::createDecrementExpression( llvm::BasicBlock* _currentBlock,
     CREATE_BINARY_OPERATOR(  _GTL_Function_Name_, v1, v2); \
   }
 
-CREATE_BINARY_EXPRESSION(createAdditionExpression, getAdd, createAdd)
-CREATE_BINARY_EXPRESSION(createSubstractionExpression, getSub, createSub)
-CREATE_BINARY_EXPRESSION(createMultiplicationExpression, getMul, createMul)
-CREATE_BINARY_EXPRESSION(createRightShiftExpression, getAShr, createAShr);
-CREATE_BINARY_EXPRESSION(createLeftShiftExpression, getShl, createShl);
+CREATE_BINARY_EXPRESSION(createAdditionExpression, getAdd, CreateAdd)
+CREATE_BINARY_EXPRESSION(createSubstractionExpression, getSub, CreateSub)
+CREATE_BINARY_EXPRESSION(createMultiplicationExpression, getMul, CreateMul)
+CREATE_BINARY_EXPRESSION(createRightShiftExpression, getAShr, CreateAShr)
+CREATE_BINARY_EXPRESSION(createLeftShiftExpression, getShl, CreateShl)
 
 
 llvm::Value* CodeGenerator::createDivisionExpression(llvm::BasicBlock* currentBlock, llvm::Value* lhs, const Type* lhsType, llvm::Value* rhs, const Type* rhsType)
@@ -364,9 +411,9 @@ llvm::Value* CodeGenerator::createDivisionExpression(llvm::BasicBlock* currentBl
       or ( lhsType->dataType() == GTLCore::Type::VECTOR
            and lhsType->embeddedType()->dataType() == GTLCore::Type::FLOAT ) )
   {
-    return llvm::BinaryOperator::createFDiv(v1, v2, "", currentBlock );
+    return llvm::BinaryOperator::CreateFDiv(v1, v2, "", currentBlock );
   } else {
-    return llvm::BinaryOperator::createSDiv(v1, v2, "", currentBlock );
+    return llvm::BinaryOperator::CreateSDiv(v1, v2, "", currentBlock );
   }
 }
 
@@ -392,7 +439,7 @@ ExpressionResult CodeGenerator::createDivisionExpression( llvm::BasicBlock* curr
 llvm::Value* CodeGenerator::createModuloExpression(llvm::BasicBlock* currentBlock, llvm::Value* lhs, const Type* lhsType, llvm::Value* rhs, const Type* rhsType)
 {
   UNIFORMIZE_VALUE_TYPES( lhs, rhs );
-  return llvm::BinaryOperator::createSRem( v1, v2, "", currentBlock );
+  return llvm::BinaryOperator::CreateSRem( v1, v2, "", currentBlock );
 }
 
 llvm::Constant* CodeGenerator::createModuloExpression( llvm::Constant* lhs, const Type* lhsType, llvm::Constant* rhs, const Type* rhsType)
@@ -425,8 +472,8 @@ ExpressionResult CodeGenerator::createModuloExpression( llvm::BasicBlock* curren
     CREATE_BINARY_OPERATOR(  _GTL_Function_Name_, v1, v2); \
   }
 
-CREATE_BINARY_EXPRESSION(createOrExpression, getOr, createOr)
-CREATE_BINARY_EXPRESSION(createAndExpression, getAnd, createAnd)
+CREATE_BINARY_EXPRESSION(createOrExpression, getOr, CreateOr)
+CREATE_BINARY_EXPRESSION(createAndExpression, getAnd, CreateAnd)
 
 #define CREATE_BINARY_INTEGER_EXPRESSION(_GTL_Function_Name_, _LLVM_Constant_Expr_, _LLVM_Binary_Operator ) \
   llvm::Value* CodeGenerator::_GTL_Function_Name_(llvm::BasicBlock* currentBlock, llvm::Value* lhs, const Type* lhsType, llvm::Value* rhs, const Type* rhsType) \
@@ -449,9 +496,9 @@ CREATE_BINARY_EXPRESSION(createAndExpression, getAnd, createAnd)
     CREATE_BINARY_OPERATOR(  _GTL_Function_Name_, v1, v2); \
   }
 
-CREATE_BINARY_INTEGER_EXPRESSION(createBitXorExpression, getXor, createXor )
-CREATE_BINARY_INTEGER_EXPRESSION(createBitOrExpression, getOr, createOr )
-CREATE_BINARY_INTEGER_EXPRESSION(createBitAndExpression, getAnd, createAnd )
+CREATE_BINARY_INTEGER_EXPRESSION(createBitXorExpression, getXor, CreateXor )
+CREATE_BINARY_INTEGER_EXPRESSION(createBitOrExpression, getOr, CreateOr )
+CREATE_BINARY_INTEGER_EXPRESSION(createBitAndExpression, getAnd, CreateAnd )
 
 
 #define CREATE_COMPARISON_EXPRESSION(_GTL_Function_Name_, _LLVM_UInt_Expr_, _LLVM_SInt_Expr_, _LLVM_Float_Expr_ ) \
@@ -528,7 +575,7 @@ ExpressionResult CodeGenerator::createMinusExpression( llvm::BasicBlock* current
 
 llvm::Value* CodeGenerator::createMinusExpression(llvm::BasicBlock* currentBlock, llvm::Value* rhs, const Type* rhsType)
 {
-  return llvm::BinaryOperator::createNeg( rhs, "", currentBlock );
+  return llvm::BinaryOperator::CreateNeg( rhs, "", currentBlock );
 }
 
 llvm::Constant* CodeGenerator::createMinusExpression( llvm::Constant* rhs, const Type* rhsType)
@@ -543,7 +590,7 @@ ExpressionResult CodeGenerator::createNotExpression( llvm::BasicBlock* currentBl
 
 llvm::Value* CodeGenerator::createNotExpression(llvm::BasicBlock* currentBlock, llvm::Value* rhs, const Type* rhsType)
 {
-  return llvm::BinaryOperator::createNot( rhs, "", currentBlock );
+  return llvm::BinaryOperator::CreateNot( rhs, "", currentBlock );
 }
 
 llvm::Constant* CodeGenerator::createNotExpression( llvm::Constant* rhs, const Type* rhsType)
@@ -558,7 +605,7 @@ ExpressionResult CodeGenerator::createTildeExpression( llvm::BasicBlock* current
 
 llvm::Value* CodeGenerator::createTildeExpression(llvm::BasicBlock* currentBlock, llvm::Value* rhs, const Type* rhsType)
 {
-  return llvm::BinaryOperator::createXor( rhs, integerToConstant(0xFFFFFFFF ), "", currentBlock );
+  return llvm::BinaryOperator::CreateXor( rhs, integerToConstant(0xFFFFFFFF ), "", currentBlock );
 }
 
 llvm::Constant* CodeGenerator::createTildeExpression( llvm::Constant* rhs, const Type* rhsType)
@@ -571,7 +618,7 @@ llvm::Value* CodeGenerator::accessArrayValue( llvm::BasicBlock* _currentBlock, l
   GTL_DEBUG( *_pointer << " " << *_index );
   std::vector<llvm::Value*> indexes;
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the structure of the array
-  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 1)); // Access the size of the array
+  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, ArrayWrap::POS_DATA)); // Access the data of the array
   
   // Select the pointer
   llvm::Value* iptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
@@ -607,11 +654,12 @@ GTLCore::ExpressionResult CodeGenerator::callFunction( GenerationContext& _gc, l
       llvm::Value* value = 0;
       if( (*it)->type()->dataType() == GTLCore::Type::ARRAY or (*it)->type()->dataType() == GTLCore::Type::STRUCTURE )
       {
-        if( AST::AccessorExpression* aexp = dynamic_cast<AST::AccessorExpression*>( *it ) )
+        AST::AccessorExpression* aexp = dynamic_cast<AST::AccessorExpression*>( *it );
+        if( aexp and not dynamic_cast<AST::FunctionMemberAccessorExpression*>( aexp ) )
         {
           value = aexp->pointer( _gc, bb );
         } else if( dynamic_cast<AST::FunctionCallExpression*>( *it )
-                   or dynamic_cast<AST::GlobalDataExpression*>( *it ) ) {
+                   or dynamic_cast<AST::GlobalDataExpression*>( *it ) or dynamic_cast<AST::FunctionMemberAccessorExpression*>( aexp ) ) {
             value = (*it)->generateValue(_gc, bb).value();
         } else {
           GTL_ABORT("Can't pass an array or a structure");
@@ -621,10 +669,10 @@ GTLCore::ExpressionResult CodeGenerator::callFunction( GenerationContext& _gc, l
       }
       GTL_ASSERT( value );
       GTL_DEBUG( "param's type = " << *oparam_it->type()->d->type() << " value = " << *value );
-      if( oparam_it->type()->d->type()->isFirstClassType() )
-      {
-        value = _gc.codeGenerator()-> convertValueTo( bb, value, (*it)->type(), oparam_it->type() );
-      }
+//       if( oparam_it->type()->d->type()->isFirstClassType() )
+//       {
+//         value = _gc.codeGenerator()-> convertValueTo( bb, value, (*it)->type(), oparam_it->type() );
+//       }
       convertedParams.push_back( value );
     }
     
@@ -650,6 +698,101 @@ GTLCore::ExpressionResult CodeGenerator::callFunction( GenerationContext& _gc, l
 #endif
   
   llvm::CallInst *CallFunc = llvm::CallInst::Create(llvmFunction, convertedParams.begin(), convertedParams.end(), "", bb);
-  CallFunc->setTailCall();
+  CallFunc->setTailCall(false);
   return GTLCore::ExpressionResult( CallFunc, _function->returnType(), true );
 }
+
+llvm::Value* CodeGenerator::countFieldPointer( llvm::BasicBlock* _currentBlock, llvm::Value* _pointer )
+{
+  std::vector<llvm::Value*> indexes;
+  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the structure of the array/structure
+  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the count field of the array/structure
+  // Select the pointer
+  return llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
+}
+
+void CodeGenerator::setCountFieldOf( llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, llvm::Value* _value )
+{
+  new llvm::StoreInst( _value, countFieldPointer( _currentBlock, _pointer ), "", _currentBlock);
+}
+
+llvm::Value* CodeGenerator::getCountFieldOf( llvm::BasicBlock* _currentBlock, llvm::Value* _pointer )
+{
+  return new llvm::LoadInst( countFieldPointer( _currentBlock, _pointer ), "", _currentBlock );
+}
+
+llvm::Value* CodeGenerator::incrementCountFieldOf( llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, llvm::Value* _increment )
+{
+  llvm::Value* pointerCF = countFieldPointer(_currentBlock, _pointer);
+  llvm::Value* val = createAdditionExpression( _currentBlock,
+                                new llvm::LoadInst( pointerCF, "CodeGenerator::incrementCountFieldOf", _currentBlock ),
+                                Type::Integer32, _increment, Type::Integer32 );
+  new llvm::StoreInst( val, pointerCF, "CodeGenerator::incrementCountFieldOf", _currentBlock );
+  return val;
+}
+
+#ifdef _USE_GTL_MM_
+
+llvm::Function* createGtlMallocFunction( llvm::Module* _module )
+{
+  std::vector<const llvm::Type*> memcpyTyArgs;
+  memcpyTyArgs.push_back(llvm::Type::Int32Ty);
+  llvm::FunctionType* memcpyTy = llvm::FunctionType::get( llvm::PointerType::get(llvm::IntegerType::get(8), 0), memcpyTyArgs, false);
+  
+  llvm::Function* func_llvm_memcpy_i32 = (llvm::Function*)_module->getOrInsertFunction(
+      "gtl_malloc", memcpyTy); // (external, no body)
+  func_llvm_memcpy_i32->setCallingConv(llvm::CallingConv::C);
+  return func_llvm_memcpy_i32;
+}
+
+llvm::Function* createGtlFreeFunction( llvm::Module* _module )
+{
+  std::vector<const llvm::Type*> memcpyTyArgs;
+  memcpyTyArgs.push_back(llvm::PointerType::get(llvm::IntegerType::get(8), 0));
+  llvm::FunctionType* memcpyTy = llvm::FunctionType::get( llvm::Type::VoidTy, memcpyTyArgs, false);
+  
+  llvm::Function* func_llvm_memcpy_i32 = (llvm::Function*)_module->getOrInsertFunction(
+      "gtl_free", memcpyTy); // (external, no body)
+  func_llvm_memcpy_i32->setCallingConv(llvm::CallingConv::C);
+  return func_llvm_memcpy_i32;
+}
+#endif
+
+llvm::Value* CodeGenerator::allocateMemory( GenerationContext& _gc, const llvm::Type* _type, llvm::Value* _size, llvm::BasicBlock* _bb)
+{
+#ifdef _USE_GTL_MM_
+  llvm::Function* func = createGtlMallocFunction( _gc.module()->llvmModule() );
+  std::vector<llvm::Value*> params;
+  GTL_DEBUG( *_size << "  * " << *llvm::ConstantExpr::getSizeOf( _type ) );
+  params.push_back( llvm::BinaryOperator::CreateMul( 
+    _size,
+    llvm::CastInst::CreateIntegerCast( llvm::ConstantExpr::getSizeOf( _type ),
+                                       llvm::Type::Int32Ty, false, "", _bb ), "", _bb  ) );
+  GTL_DEBUG( *func );
+  llvm::CallInst *CallFunc = llvm::CallInst::Create(func, params.begin(), params.end(), "", _bb);
+  CallFunc->setTailCall(false);
+  return convertPointerTo( _bb, CallFunc, _type );
+#else
+#ifndef _USE_OS_MM_
+#error _USE_OS_MM_ or _USE_GTL_MM_ should be defined
+#endif
+  return new llvm::MallocInst( _type, _size, "", _bb);
+#endif
+}
+
+void CodeGenerator::freeMemory( GenerationContext& _gc, llvm::Value* _ptr, llvm::BasicBlock* _bb)
+{
+#ifdef _USE_GTL_MM_
+  llvm::Function* func = createGtlFreeFunction( _gc.module()->llvmModule() );
+  std::vector<llvm::Value*> params;
+  params.push_back( convertPointerTo( _bb, _ptr, llvm::IntegerType::get(8) ) );
+  GTL_DEBUG(*params[0]);
+  llvm::CallInst *CallFunc = llvm::CallInst::Create(func, params.begin(), params.end(), "", _bb);
+  CallFunc->setTailCall(false);
+#else
+#ifndef _USE_OS_MM_
+#error _USE_OS_MM_ or _USE_GTL_MM_ should be defined
+#endif
+  new llvm::FreeInst( _ptr, _bb );
+#endif
+}
