@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,6 +22,7 @@
 
 namespace OpenCTL {
   class Compiler;
+  class Program;
 }
 
 namespace OpenShiva {
@@ -29,9 +30,15 @@ namespace OpenShiva {
 }
 
 namespace GTLCore {
+  /**
+   * This class contains information about the current level of optimization.
+   * @ingroup GTLCore
+   */
   class Optimiser{
     friend class OpenCTL::Compiler;
+    friend class OpenCTL::Program;
     friend class OpenShiva::Compiler;
+    friend class ModuleData;
     private:
       Optimiser();
       ~Optimiser();
@@ -42,6 +49,10 @@ namespace GTLCore {
        * @param _level from 0 to 4 (0 means no optimisation, 4 is aggressive optimisation)
        */
       void setLevel(int _level);
+      /**
+       * @return the maximum level that can be used without knwown problems
+       */
+      static int maximumRecommendedLevel();
     private:
       class Private;
       Private* const d;
