@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,6 +34,7 @@
 
 #include "AST/AccessorExpression.h"
 #include "AST/BinaryExpression.h"
+#include "AST/ConstantCoumpoundExpression.h"
 #include "AST/CoumpoundExpression.h"
 #include "AST/Expression.h"
 #include "AST/FunctionDeclaration.h"
@@ -46,24 +47,19 @@
 #include "Token_p.h"
 #include "Function_p.h"
 #include "Utils_p.h"
+#include "VariablesManager_p.h"
 
-#include "TypeManager.h"
-#include "TypeManager_p.h"
+#include "TypesManager.h"
+#include "TypesManager_p.h"
 
 using namespace GTLCore;
 
-struct ParserBase::Context
-{
-  std::map< ScopedName, VariableNG* > variables;
-};
-
 struct ParserBase::Private {
   Token currentToken;
   CompilerBase* compiler;
   LexerBase* lexer;
   String nameSpace;
-  std::list< Context > contextes;
-  std::list<VariableNG*> functionVariables;
+  VariablesManager variablesManager;
 };
 
 ParserBase::ParserBase(CompilerBase* _compiler, LexerBase* _lexer) : d(new Private)
@@ -87,6 +83,67 @@ const Token& ParserBase::currentToken()
   return d->currentToken;
 }
 
+
+void ParserBase::parseConstantDeclaration()
+{
+  getNextToken();
+  const GTLCore::Type* type = parseType();
+  if( isOfType( currentToken(), GTLCore::Token::IDENTIFIER ) )
+  {
+    GTLCore::String name = currentToken().string;
+    getNextToken();
+    // Decode the size of array (if applicable)
+    std::list<int> memberArraySize = expressionsListToIntegersList( parseArraySize(true) );
+    if( d->compiler) // This is needed for the Parser's test
+    {
+      type = d->compiler->typesManager()->getArray( type, memberArraySize.size() );
+    }
+    // Check that there is an initialiser
+    if( isOfType( currentToken(), GTLCore::Token::EQUAL ) )
+    {
+      getNextToken();
+      AST::Expression* expression = 0 ;
+      // Check if it's a coumpound expression
+      if( currentToken().type == GTLCore::Token::STARTBRACE )
+      {
+        GTLCore::AST::Expression* coumpoundExpression = parseCoumpoundExpression( type, true );
+      /* if( coumpoundExpression->size() != *memberArraySize.begin()) // TODO: recurisvely check other sizes
+        {
+          GTL_DEBUG( memberArraySize.size() << " " << coumpoundExpression->size() );
+          reportError("Wrong coumpound initialiser size", currentToken() );
+        }*/
+        expression = coumpoundExpression;
+      } else {
+        expression = parseExpression(true);
+      }
+      if( isOfType( currentToken(), GTLCore::Token::SEMI ) )
+      {
+        getNextToken();
+        if(expression)
+        {
+          GTLCore::ScopedName scopedName( nameSpace(), name );
+          if( tree()->containsGlobalConstant( scopedName ) )
+          {
+            reportError("Constant '" + scopedName.toString() + "' has allready been declared", currentToken());
+          } else {
+            AST::GlobalConstantDeclaration* gcd = new AST::GlobalConstantDeclaration( scopedName, type , expression, false );
+            variablesManager()->declareConstant( scopedName, gcd->variable() );
+            tree()->append( gcd );
+          }
+        }
+      }
+    } else {
+      GTL_DEBUG("Unexpected");
+      reportUnexpected( currentToken() );
+      reachNextSemi();
+    }
+  } else {
+    GTL_DEBUG("Unexpected");
+    reportUnexpected( currentToken() );
+    reachNextSemi();
+  }
+}
+
 void ParserBase::parseStructDefinition()
 {
   GTL_ASSERT( d->currentToken.type == Token::STRUCT );
@@ -95,7 +152,7 @@ void ParserBase::parseStructDefinition()
   {
     String name = d->currentToken.string;
     getNextToken();
-    if( d->compiler->typeManager()->d->isKnownType( name ) )
+    if( d->compiler->typesManager()->d->isKnownType( name ) )
     { // It has allready been declared
       reportError( name + " has allready been declared", d->currentToken);
     } else {
@@ -115,7 +172,7 @@ void ParserBase::parseStructDefinition()
               getNextToken();
               // Check if it's an array
               std::list<int> memberArraySize = expressionsListToIntegersList( parseArraySize(true) );
-              smtype = d->compiler->typeManager()->getArray( smtype, memberArraySize.size() );
+              smtype = d->compiler->typesManager()->getArray( smtype, memberArraySize.size() );
               // Check if expression end by a SEMI and add the struct member
               if( isOfType( d->currentToken, Token::SEMI ) )
               {
@@ -140,15 +197,81 @@ void ParserBase::parseStructDefinition()
         if( isOfType( d->currentToken, Token::SEMI ) )
         {
           getNextToken();
-          d->compiler->typeManager()->d->createStructure( name, members);
+          d->compiler->typesManager()->d->createStructure( name, members);
         }
       }
     }
   }
 }
 
+AST::Expression* ParserBase::parseMemberArrayConstantExpression(AST::Expression* _expression)
+{
+  if( d->currentToken.type == Token::DOT or d->currentToken.type == Token::STARTBOXBRACKET )
+  {
+    // Check we have a coumpound expression
+    AST::ConstantCoumpoundExpression* coumpound = dynamic_cast<AST::ConstantCoumpoundExpression*>( _expression );
+    if( not coumpound )
+    {
+      getNextToken();
+      reportError( "Expected value of array, structure or vector type.", d->currentToken );
+      return 0;
+    }
+    // Compute the idx
+    int idx = -1;
+    if( d->currentToken.type == Token::DOT )
+    {
+      getNextToken();
+      if( d->currentToken.type == Token::SIZE )
+      {
+        return new AST::NumberExpression<int>( coumpound->size() );
+      } else if( isOfType( d->currentToken, Token::IDENTIFIER ) ) {
+        String name = d->currentToken.string;
+        getNextToken();
+        if( _expression->type()->dataType() == Type::STRUCTURE ) {
+          idx = structMemberNameToIdx( _expression->type(), name );
+        } else if( _expression->type()->dataType() == Type::VECTOR ) {
+          idx = vectorMemberNameToIdx( name );
+        } 
+      }
+    } else if( d->currentToken.type == Token::STARTBOXBRACKET )
+    {
+      getNextToken();
+      AST::Expression* expr = parseExpression( true );
+      GTL_ASSERT( expr->isConstant() );
+      expr = d->compiler->convertCenter()->createConvertExpression( expr, GTLCore::Type::Integer32 );
+      if( expr and expr->isConstant() )
+      {
+        GenerationContext gc( 0, 0, 0 ,0 );
+        llvm::ConstantInt* idxCST = dynamic_cast<llvm::ConstantInt*>( expr->generateValue( gc, 0).constant() );
+        GTL_ASSERT( idxCST );
+        idx = idxCST->getLimitedValue();
+      } else {
+        reportError( "Expected integer constant", d->currentToken );
+        return 0;
+      }
+      if( isOfType( d->currentToken, Token::ENDBOXBRACKET ) )
+      {
+        getNextToken();
+      }
+    } else {
+      GTL_ABORT("Impossible error.");
+    }
+    GTL_DEBUG( idx );
+    if( idx >= 0 and idx < coumpound->size() )
+    {
+      return parseMemberArrayConstantExpression( coumpound->expressionAt( idx ) );
+    } else {
+      reportError( "Invalid index.", d->currentToken );
+      return 0;
+    }
+  } else {
+    return _expression;
+  }
+}
+
 AST::AccessorExpression* ParserBase::parseMemberArrayExpression(AST::AccessorExpression* _expression, bool _constantExpression)
 {
+  GTL_ASSERT( not _constantExpression);
   if( d->currentToken.type == Token::DOT )
   {
     getNextToken();
@@ -167,19 +290,16 @@ AST::AccessorExpression* ParserBase::parseMemberArrayExpression(AST::AccessorExp
         {
           reportError("Unknown member: '" + name + "' for structure " + _expression->type()->structName(), d->currentToken );
         } else {
-          std::list<AST::Expression*> arguments = parseArguments( sfm->name(), sfm->parameters() );
+          std::list<AST::Expression*> arguments = parseArguments( sfm->name(), sfm->parameters(), 1 );
           return new AST::FunctionMemberAccessorExpression(_expression, sfm, arguments);
         }
+      } else if( _expression->type()->dataType() == Type::STRUCTURE ) {
+        return parseMemberArrayExpression( new AST::StructAccessorExpression( _expression ,  structMemberNameToIdx( _expression->type(), name ) ), _constantExpression);
+      } else if( _expression->type()->dataType() == Type::VECTOR ) {
+        return new AST::ArrayAccessorExpression( _expression, new AST::NumberExpression<int>( vectorMemberNameToIdx( name ) ) );
       } else {
-        int index = _expression->type()->d->memberToIndex( name );
-        if( index == -1 )
-        {
-          reportError("Unknown member: '" + name + "' for structure " + _expression->type()->structName(), d->currentToken );
-          delete _expression;
-          return 0;
-        } else {
-          return parseMemberArrayExpression( new AST::StructAccessorExpression( _expression ,  index ), _constantExpression);
-        }
+        GTL_DEBUG("unexpected");
+        reportUnexpected( d->currentToken );
       }
     }
   } else if( d->currentToken.type == Token::STARTBOXBRACKET )
@@ -214,6 +334,7 @@ std::list< AST::Expression* > ParserBase::parseArraySize(bool _constantExpressio
         getNextToken();
         list_.push_back( expr );
       } else {
+        GTL_DEBUG("unexpected");
         reportUnexpected( d->currentToken );
         list_.push_back( 0 );
       }
@@ -222,49 +343,102 @@ std::list< AST::Expression* > ParserBase::parseArraySize(bool _constantExpressio
   return list_;
 }
 
-AST::CoumpoundExpression* ParserBase::parseCoumpoundExpression( const Type* _type, bool _constantExpression )
+AST::Expression* ParserBase::parseCoumpoundExpression( const Type* _type, bool _constantExpression )
 {
   GTL_ASSERT( _type );
-  GTL_ASSERT( d->currentToken.type == Token::STARTBRACE );
+  GTL_DEBUG( *_type );
+  
+  Token::Type endToken;
+  switch( d->currentToken.type)
+  {
+    case Token::STARTBRACE:
+      endToken = Token::ENDBRACE;
+      break;
+    case Token::STARTBRACKET:
+      endToken = Token::ENDBRACKET;
+      break;
+    default:
+      GTL_DEBUG("unexpected");
+      reportUnexpected( d->currentToken );
+      return 0;
+  }
+  
   getNextToken(); // Eat '{'
   std::vector< AST::Expression* > expressions_;
   int index = 0;
   while(true)
   {
     AST::Expression* expression = 0;
+    const Type* subtype = _type->d->subtypeAt( index );
     if( d->currentToken.type == Token::STARTBRACE )
     {
-      const Type* type = 0;
-      if ( _type->dataType() == Type::ARRAY )
-      {
-        type = _type->embeddedType();
-      } else if( _type->dataType() == Type::STRUCTURE )
-      {
-        type = (*_type->structDataMembers())[index].type();
-      }
-      GTL_ASSERT( type );
-      if( type->dataType() != Type::STRUCTURE and type->dataType() != Type::ARRAY )
+      GTL_ASSERT( subtype );
+      GTL_DEBUG( index << " " <<  *subtype );
+      if( subtype->dataType() != Type::STRUCTURE and subtype->dataType() != Type::ARRAY )
       {
+        GTL_DEBUG("unexpected");
         reportUnexpected( d->currentToken );
         return 0;
       }
-      expression = parseCoumpoundExpression( type, _constantExpression );
+      expression = parseCoumpoundExpression( subtype, _constantExpression );
     } else {
       expression = parseExpression( _constantExpression );
     }
+    if( not expression )
+    {
+      deleteAll( expressions_ );
+      return 0;
+    }
+    expression = d->compiler->convertCenter()->createConvertExpression( expression, subtype );
+    if( not expression )
+    {
+      reportError( "can't convert coumpound expression at " + String::number( index ) + ".", d->currentToken);
+      return 0;
+    }
     expressions_.push_back( expression );
     if( d->currentToken.type == Token::COMA )
     {
       getNextToken();
-    } else if( d->currentToken.type == Token::ENDBRACE )
+    } else if( d->currentToken.type == endToken )
     {
       getNextToken();
-      return new AST::CoumpoundExpression(_type, expressions_);
+      if( _constantExpression )
+      {
+        return new AST::ConstantCoumpoundExpression(_type, expressions_);
+      } else {
+        return new AST::CoumpoundExpression(_type, expressions_);
+      }
     }
     ++index;
   }
 }
 
+AST::Expression* ParserBase::parseTypeCoumpoundExpression(bool _constantExpression )
+{
+  const Type* type = parseType();
+  if( isOfType( d->currentToken, Token::STARTBRACKET ) )
+  {
+    AST::Expression* expression = 0;
+    if( type->dataType() == Type::VECTOR or type->dataType() == Type::ARRAY or type->dataType() == Type::STRUCTURE )
+    {
+      expression = parseCoumpoundExpression( type, _constantExpression);
+    } else {
+      getNextToken();
+      expression = parseExpression(_constantExpression, type);
+      isOfType( currentToken(), Token::ENDBRACKET );
+      getNextToken();
+    }
+    if( expression )
+    {
+      return d->compiler->convertCenter()->createConvertExpression( expression, type );
+    }
+  } else {
+    GTL_DEBUG("unexpected");
+    reportUnexpected( d->currentToken );
+  }
+  return 0;
+}
+
 AST::Expression* ParserBase::parseExpression(bool _constantExpression, const GTLCore::Type* _type)
 {
   // The first token of an expression is either a primary, or an unary expression or a parenthesis
@@ -281,29 +455,6 @@ AST::Expression* ParserBase::parseExpression(bool _constantExpression, const GTL
       }
     }
     return expr;
-  } else if( d->currentToken.isPrimary() )
-  {
-    GTL_DEBUG( d->currentToken.isPrimary() << " " << d->currentToken);
-    AST::Expression* exprConst = parsePrimaryExpression(_constantExpression);
-    if( exprConst )
-    { // What's next ? A binary operator or a ;
-      if( d->currentToken.isExpressionTerminal() )
-      {
-        return exprConst;
-      } else if( d->currentToken.isBinaryOperator() ) {
-        return parseFlatBinaryOperator( _constantExpression, exprConst );
-      } else {
-        GTL_DEBUG("unexpected");
-        reportUnexpected( d->currentToken );
-        getNextToken();
-        delete exprConst;
-        return 0;
-      }
-    } else {
-      // An error has occured
-      reportError("Parse error while parsing constant", d->currentToken );
-      return 0;
-    }
   } else if( d->currentToken.isUnaryOperator() )
   {
     AST::Expression* expr = parseUnaryOperator(_constantExpression);
@@ -316,14 +467,43 @@ AST::Expression* ParserBase::parseExpression(bool _constantExpression, const GTL
     }
   } else if( currentToken().type == Token::STARTBRACE )
   {
-    AST::CoumpoundExpression* coumpoundExpression = parseCoumpoundExpression( _type, true );
+    AST::Expression* coumpoundExpression = parseCoumpoundExpression( _type, true );
     if( coumpoundExpression )
     {
       return new AST::GlobalDataExpression( coumpoundExpression );
     }
   } else {
+    AST::Expression* expression;
+    if( d->currentToken.isPrimary() )
+    {
+      GTL_DEBUG( d->currentToken.isPrimary() << " " << d->currentToken);
+      expression = parsePrimaryExpression(_constantExpression);
+      if(not expression)
+      {
+        // An error has occured
+        reportError("Parse error while parsing constant", d->currentToken );
+        return 0;
+      }
+    } else if( isType(currentToken()) ) {
+      expression = parseTypeCoumpoundExpression( _constantExpression );
+    } else {
+      GTL_DEBUG("unexpected");
+      reportUnexpected( d->currentToken );
+      return 0;
+    }
+    if( expression )
+    { // What's next ? A binary operator or a ;
+      if( d->currentToken.isExpressionTerminal() )
+      {
+        return expression;
+      } else if( d->currentToken.isBinaryOperator() ) {
+        return parseFlatBinaryOperator( _constantExpression, expression );
+      }
+    }
     GTL_DEBUG("unexpected");
     reportUnexpected( d->currentToken );
+    getNextToken();
+    delete expression;
     return 0;
   }
   return 0;
@@ -348,7 +528,7 @@ AST::Expression* ParserBase::parseFlatBinaryOperator( bool _constantExpression,
 
 AST::Expression* ParserBase::parseBinaryOperator( bool _constantExpression, AST::Expression* _lhs)
 {
-  GTL_ASSERT(_lhs);
+  if( not _lhs) return 0;
   Token binOp = d->currentToken;
   GTL_DEBUG( "parseBinaryOperator: " << binOp )
   getNextToken();
@@ -375,11 +555,13 @@ AST::Expression* ParserBase::parseBinaryOperator( bool _constantExpression, AST:
       } else if( d->currentToken.isUnaryOperator() ) {
         expr = parseUnaryOperator(_constantExpression);
       } else if( currentToken().type == Token::STARTBRACE ) {
-        AST::CoumpoundExpression* coumpoundExpression = parseCoumpoundExpression( _lhs->type() , true );
+        AST::Expression* coumpoundExpression = parseCoumpoundExpression( _lhs->type() , true );
         if( coumpoundExpression )
         {
-          return new AST::GlobalDataExpression( coumpoundExpression );
+          expr = new AST::GlobalDataExpression( coumpoundExpression );
         }
+      } else if( isType(currentToken()) ) {
+        expr = parseTypeCoumpoundExpression(_constantExpression);      
       } else {
         GTL_DEBUG("unexpected");
         reportUnexpected( d->currentToken );
@@ -598,13 +780,14 @@ void ParserBase::parseFunction()
   if( not returnType ) return;
   
   // Clear the list of variables
-  d->functionVariables.clear();
+  d->variablesManager.reset();
   
   // Next is the name of the function
   if( isOfType( d->currentToken, Token::IDENTIFIER ) )
   {
     Token nameToken = d->currentToken;
     ScopedName name( d->nameSpace, d->currentToken.string );
+    startParsingFunction( name.name() );
     getNextToken();
     // Next is start bracket
     if( isOfType( d->currentToken, Token::STARTBRACKET ) )
@@ -644,16 +827,23 @@ void ParserBase::parseFunction()
               getNextToken();
               AST::Expression* initialiser = 0;
               std::list<AST::Expression*> memberArraySize = parseArraySize(true);
-              ptype = d->compiler->typeManager()->getArray( ptype, memberArraySize.size() );
+              ptype = d->compiler->typesManager()->getArray( ptype, memberArraySize.size() );
               deleteAll( memberArraySize);
               if( d->currentToken.type == Token::EQUAL )
               {
                 getNextToken();
-                initialiser = parseExpression( false, ptype );
+                initialiser = d->compiler->convertCenter()->createConvertExpression( parseExpression( false, ptype ), ptype);
+                if( not initialiser )
+                {
+                  reportError( "Can't convert initialiser expression to the parameter type, for parameter " + String::number((unsigned int)(params.size() + 1)) + ".", d->currentToken);
+                  deleteAll( params );
+                  return;
+                }
                 needToHaveInitialiser = true;
               } else if( needToHaveInitialiser )
               {
                 reportError( "Parameter need to have default initialiser once a parameter on the left had one.", d->currentToken );
+                deleteAll( params );
                 return;
               }
               if( d->currentToken.type == Token::COMA or d->currentToken.type == Token::ENDBRACKET )
@@ -685,11 +875,13 @@ void ParserBase::parseFunction()
               } else {
                 GTL_DEBUG("Unexpected");
                 reportUnexpected( d->currentToken );
+                deleteAll( params );
                 return;
               }
             } else {
               GTL_DEBUG("Unexpected");
               reportUnexpected( d->currentToken );
+              deleteAll( params );
               return;
             }
           } else {
@@ -703,14 +895,14 @@ void ParserBase::parseFunction()
           if( isOfType( d->currentToken, Token::STARTBRACE ) )
           {
             AST::FunctionDeclaration* fd = new AST::FunctionDeclaration( name, returnType, params );
-            startContext();
             // TODO insert functions arguments in the context
             for( unsigned int i = 0; i < params.size(); ++i)
             {
-              d->contextes.begin()->variables[ ScopedName("",params[i]->parameter().name() )] = fd->parametersVariable()[i];
+              d->variablesManager.declareParameter( ScopedName("",params[i]->parameter().name() ), fd->parametersVariable()[i] );
             }
-            AST::Statement* statement = parseStatementList();
-            endContext();
+            d->variablesManager.startContext();
+            AST::Statement* statement =appendCurrentContextGarbageCollecting(parseStatementList());
+            d->variablesManager.endContext();
             if( statement )
             {
               bool v = d->compiler->declareFunction( name, fd->function() );
@@ -732,10 +924,32 @@ void ParserBase::parseFunction()
         }
       }
     }
+    endParsingFunction( name.name() );
   }
 }
 
-AST::Statement* ParserBase::parseStatementList()
+AST::StatementsList* ParserBase::appendCurrentContextGarbageCollecting( AST::StatementsList* _statementsList )
+{
+  if( _statementsList and not _statementsList->isReturnStatement() )
+  {
+    _statementsList->appendStatement( d->variablesManager.garbageCollectCurrentContext() );
+  }
+  return _statementsList;
+}
+
+AST::Statement* ParserBase::appendCurrentContextGarbageCollecting( AST::Statement* _statement )
+{
+  if( _statement and not _statement->isReturnStatement() )
+  {
+    std::list< AST::Statement* > statements;
+    statements.push_back( _statement );
+    statements.push_back( d->variablesManager.garbageCollectCurrentContext() );
+    return new AST::StatementsList( statements );
+  }
+  return _statement;
+}
+
+AST::StatementsList* ParserBase::parseStatementList()
 {
   GTL_ASSERT( d->currentToken.type == Token::STARTBRACE );
   getNextToken();
@@ -791,15 +1005,15 @@ AST::Statement* ParserBase::parsePrintStatement()
 
 AST::Statement* ParserBase::parseStatementOrList()
 {
-  startContext();
+  d->variablesManager.startContext();
   AST::Statement* statement;
   if( d->currentToken.type == Token::STARTBRACE )
   {
-    statement = parseStatementList();
+    statement = appendCurrentContextGarbageCollecting( parseStatementList() );
   } else {
-    statement = parseStatement();
+    statement = appendCurrentContextGarbageCollecting( parseStatement() );
   }
-  endContext();
+  d->variablesManager.endContext();
   return statement;
 }
 
@@ -823,7 +1037,7 @@ AST::Statement* ParserBase::parseWhileStatement()
 
 AST::Statement* ParserBase::parseForStatement()
 {
-  startContext();
+  d->variablesManager.startContext();
   GTL_ASSERT( d->currentToken.type == Token::FOR );
   getNextToken();
   if( isOfType( d->currentToken, Token::STARTBRACKET ) )
@@ -848,8 +1062,8 @@ AST::Statement* ParserBase::parseForStatement()
       if( isOfType( d->currentToken, Token::ENDBRACKET ) )
       {
         getNextToken(); // eat the ')'
-        AST::Statement* statement = parseStatementOrList();
-        endContext();
+        AST::Statement* statement = appendCurrentContextGarbageCollecting(parseStatementOrList());
+        d->variablesManager.endContext();
         return new AST::ForStatement(initexpression, comparexpression, updateexpression, statement);
       }
       delete updateexpression;
@@ -857,7 +1071,7 @@ AST::Statement* ParserBase::parseForStatement()
     }
     delete initexpression;
   }
-  endContext();
+  d->variablesManager.endContext();
   return 0;
 }
 
@@ -893,13 +1107,13 @@ AST::Statement* ParserBase::parseReturnStatement()
   if( d->currentToken.type == Token::SEMI )
   { // TODO check type
     getNextToken(); // eat the coma
-    return new AST::ReturnStatement( 0, functionVariables() );
+    return new AST::ReturnStatement( 0, d->variablesManager.garbageCollectEverything() );
   } else {
     AST::Expression* expr = parseExpression(false);
     if(expr and isOfType( d->currentToken, Token::SEMI ) )
     {
       getNextToken();
-      return new AST::ReturnStatement( expr, functionVariables() );
+      return new AST::ReturnStatement( expr, d->variablesManager.garbageCollectEverything() );
     } else {
       return 0;
     }
@@ -934,11 +1148,11 @@ AST::Statement* ParserBase::parseVariableDeclaration()
     if( isOfType( d->currentToken, Token::IDENTIFIER ) )
     {
       ScopedName name("", d->currentToken.string);
-      if( d->contextes.begin()->variables.find( name ) == d->contextes.begin()->variables.end() )
+      if( not d->variablesManager.hasVariableInCurrentContext( name ) )
       {
         getNextToken();
         std::list<AST::Expression*> initialsize = parseArraySize(false);
-        type = d->compiler->typeManager()->getArray( type, initialsize.size() );
+        type = d->compiler->typesManager()->getArray( type, initialsize.size() );
         bool initialised = false;
         AST::Expression* initialiser = 0;
         if( d->currentToken.type == Token::EQUAL )
@@ -947,11 +1161,22 @@ AST::Statement* ParserBase::parseVariableDeclaration()
           // Check if it's a coumpound expression
           initialiser = parseExpression( false, type );
           initialised = true;
+          if( initialiser )
+          {
+            AST::Expression* previousValue = initialiser;
+            initialiser = d->compiler->convertCenter()->createConvertExpression( initialiser, type );
+            if( not initialiser )
+            {
+              delete previousValue;
+              reportError( "Can't convert value at initialisation to variable type.", d->currentToken );
+              return 0;
+            }
+          }
         }
         // Create the variable declaration statement
         AST::VariableDeclaration* variable = new AST::VariableDeclaration( type, initialiser, constant, initialsize );
         // Insert the variable in the context
-        d->contextes.begin()->variables[name] = variable->variable();
+        d->variablesManager.declareVariable( name, variable->variable());
         GTL_ASSERT( variable->variable() );
         GTL_DEBUG(initialised << " " << Token::typeToString( d->currentToken.type ) );
         if( d->currentToken.type == Token::COMA and not initialised ) {
@@ -998,7 +1223,7 @@ const Type* ParserBase::parseFunctionType()
     if( not type ) return 0;
     std::list< AST::Expression* > sizes = parseArraySize(true);
     deleteAll( sizes );
-    return d->compiler->typeManager()->getArray( type, sizes.size() );
+    return d->compiler->typesManager()->getArray( type, sizes.size() );
   }
 }
 
@@ -1027,7 +1252,7 @@ const Type* ParserBase::parseType()
     }
     case Token::IDENTIFIER:
     {
-      const Type* type = d->compiler->typeManager()->getStructure( d->currentToken.string );
+      const Type* type = d->compiler->typesManager()->getStructure( d->currentToken.string );
       if( not type )
       {
         reportError("Unknown type : " + d->currentToken.string, d->currentToken );
@@ -1075,7 +1300,7 @@ AST::Expression* ParserBase::parsePrimaryExpression(bool _constantExpression)
           {
             getNextToken();
             // Put the expr in a proxy to avoid double free when the AST tree is deleted
-            return new AST::ProxyExpression(  (*it)->initialiser() );
+            return new AST::ProxyExpression( parseMemberArrayConstantExpression( (*it)->initialiser() ) );
           }
         }
         AST::Expression* stdconst = d->compiler->standardConstant( d->currentToken.string );
@@ -1098,7 +1323,10 @@ AST::Expression* ParserBase::parsePrimaryExpression(bool _constantExpression)
         }
         if( d->currentToken.type == Token::STARTBRACKET )
         { // It's a function call
-          Function* function = d->compiler->function( name );
+          GTL_DEBUG( name );
+          ScopedName fname( ( name.nameSpace() == "" ) ? d->nameSpace : name.nameSpace(),
+                              name.name() );
+          Function* function = d->compiler->function( fname );
           if( not function )
           {
             reportError("Unknown function: " + name.toString(), d->currentToken);
@@ -1107,7 +1335,7 @@ AST::Expression* ParserBase::parsePrimaryExpression(bool _constantExpression)
           }
           getNextToken();
           // Parse arguments
-          std::list<AST::Expression*> arguments = parseArguments( function->name().toString(), function->parameters() );
+          std::list<AST::Expression*> arguments = parseArguments( function->name().toString(), function->parameters(), 0 );
           if( arguments.size() >= function->d->data->minimumParameters() and arguments.size() <= function->d->data->maximumParameters() )
           {
             return new AST::FunctionCallExpression( function, arguments ) ;
@@ -1116,7 +1344,7 @@ AST::Expression* ParserBase::parsePrimaryExpression(bool _constantExpression)
             return 0;
           }
         } else {
-          VariableNG* var = getVariable( name );
+          VariableNG* var = d->variablesManager.getVariable( name );
           if(not var)
           {
             AST::Expression* stdconst = d->compiler->standardConstant( name.name() );
@@ -1140,7 +1368,7 @@ AST::Expression* ParserBase::parsePrimaryExpression(bool _constantExpression)
   return 0;
 }
 
-std::list<AST::Expression*> ParserBase::parseArguments( const String& _name, const std::vector< Parameter >& _parameters )
+std::list<AST::Expression*> ParserBase::parseArguments( const String& _name, const std::vector< Parameter >& _parameters, int firstParameter )
 {
   std::list<AST::Expression*> arguments;
   while(true)
@@ -1148,15 +1376,26 @@ std::list<AST::Expression*> ParserBase::parseArguments( const String& _name, con
     if( d->currentToken.type == Token::ENDBRACKET )
     {
       break;
-    } else if( arguments.size() == _parameters.size() ) {
+    } else if( arguments.size() + firstParameter == _parameters.size() ) {
       // Too much arguments
       reportError( "Too many arguments for function '" + _name + "'", d->currentToken);
       return arguments;
     } else {
       int posArg = arguments.size();
-      AST::Expression* expression = parseExpression( false, _parameters[ posArg ].type() );
+      AST::Expression* expression = parseExpression( false, _parameters[ posArg + firstParameter].type() );
+      if(not expression)
+      {
+        return arguments;
+      }
+      GTL_DEBUG( *_parameters[ posArg + firstParameter ].type() << " " << *expression->type() );
+      expression = d->compiler->convertCenter()->createConvertExpression( expression, _parameters[ posArg  + firstParameter ].type() );
+      if( not expression )
+      {
+        reportError("Can't convert parameter " + String::number( posArg + 1 ), d->currentToken  );
+        return arguments;
+      }
       arguments.push_back( expression);
-      if( _parameters[ posArg ].output() )
+      if( _parameters[ posArg + firstParameter ].output() )
       {
         if( not dynamic_cast< AST::AccessorExpression* >( expression ) )
         {
@@ -1234,38 +1473,72 @@ AST::Expression* ParserBase::createBinaryOperator( const Token& token, AST::Expr
     delete rhs;
     return 0;
   }
-  if( token.type == Token::EQUAL )
+  if( token.type == Token::EQUAL or token.type == Token::PLUSEQUAL or token.type == Token::MINUSEQUAL or token.type == Token::MULTIPLYEQUAL or token.type == Token::DIVIDEEQUAL)
   {
+    rhs = d->compiler->convertCenter()->createConvertExpression( rhs, lhs->type() );
+    if( not rhs )
+    {
+      delete lhs;
+      delete rhs;
+      reportError("Can't convert right hand side value to the left hand side type.", token);
+      return 0;
+    }
     AST::AccessorExpression* ve = dynamic_cast<AST::AccessorExpression*>( lhs );
+    GTL_ASSERT( rhs->type() == ve->type() );
     if( ve )
     {
-      return new AST::AssignementBinaryExpression( ve, rhs );
+      if( token.type == Token::PLUSEQUAL )
+      {
+        rhs = new AST::AdditionBinaryExpression( new AST::ProxyExpression(ve), rhs );
+      } else if(token.type == Token::MINUSEQUAL) {
+        rhs = new AST::SubstractionBinaryExpression( new AST::ProxyExpression(ve), rhs );
+      } else if(token.type == Token::MULTIPLYEQUAL) {
+        rhs = new AST::MultiplicationBinaryExpression( new AST::ProxyExpression(ve), rhs );
+      } else if(token.type == Token::DIVIDEEQUAL) {
+        rhs = new AST::DivisionBinaryExpression( new AST::ProxyExpression(ve), rhs );
+      }
+      if( ve->isConstant() )
+      {
+        delete lhs;
+        delete rhs;
+        reportError("Left hand side of an assignement expression can't be a constant.", token);
+        return 0;
+      } else {
+        return new AST::AssignementBinaryExpression( ve, rhs );
+      }
     }
+    delete lhs;
+    delete rhs;
     reportError("Left hand side of an assignement expression must be a variable.", token);
     return 0;
     
   } else {
-    std::pair<AST::Expression*, AST::Expression*>  ce = d->compiler->convertCenter()->createConvertExpressions( lhs, rhs );
-    GTL_DEBUG( "Convert from (" << *lhs->type() << ", " << *rhs->type() << " ) to ( " << *ce.first->type() << ", " << *ce.second->type() << " )" );
-    if( not ce.first or not ce.second )
+    if( d->compiler )
     {
-      reportError( "Can do a binary operation only on two numbers", token );
-      if(ce.first)
+      std::pair<AST::Expression*, AST::Expression*>  ce = d->compiler->convertCenter()->createConvertExpressions( lhs, rhs );
+      if( not ce.first or not ce.second )
       {
-        delete ce.first;
-      } else {
-        delete lhs;
+        GTL_DEBUG( "Convert from (" << *lhs->type() << ", " << *rhs->type() << " ) failed.");
+        reportError( "Can do a binary operation only on two numbers", token );
+        if(ce.first)
+        {
+          delete ce.first;
+        } else {
+          delete lhs;
+        }
+        if(ce.second)
+        {
+          delete ce.second;
+        } else {
+          delete rhs;
+        }
+        return 0;
       }
-      if(ce.second)
-      {
-        delete ce.second;
-      } else {
-        delete rhs;
-      }
-      return 0;
+      GTL_DEBUG( "Convert from (" << *lhs->type() << ", " << *rhs->type() << " ) to ( " << *ce.first->type() << ", " << *ce.second->type() << " )" );
+      lhs = ce.first;
+      rhs = ce.second;
     }
-    lhs = ce.first;
-    rhs = ce.second;
+    GTL_ASSERT( lhs->type() == rhs->type() );
     switch( token.type )
     {
       case Token::OR:
@@ -1312,6 +1585,7 @@ AST::Expression* ParserBase::createBinaryOperator( const Token& token, AST::Expr
         return new AST::ModuloBinaryExpression( lhs, rhs );
       }
       default:
+        GTL_ABORT("Unknown operator: " << Token::typeToString( token.type ) );
         return 0;
     }
   }
@@ -1319,50 +1593,7 @@ AST::Expression* ParserBase::createBinaryOperator( const Token& token, AST::Expr
 
 bool ParserBase::isType( const Token& token )
 {
-  return token.isNativeType() or ( token.type == Token::IDENTIFIER and d->compiler->typeManager()->d->isKnownType( token.string ) );
-}
-
-// Context management
-
-void ParserBase::startContext()
-{
-  d->contextes.push_front( Context() );
-}
-
-void ParserBase::endContext()
-{
-  d->contextes.pop_front();
-}
-
-VariableNG* ParserBase::getVariable( const ScopedName& n ) const
-{
-  GTL_DEBUG("getVariable " << n );
-  for( std::list<Context>::const_iterator cit = d->contextes.begin();
-       cit != d->contextes.end(); cit++)
-  {
-    for( std::map<ScopedName, VariableNG*>::const_iterator it = cit->variables.begin();
-         it != cit->variables.end(); ++it)
-    {
-      GTL_DEBUG( " storage: " << it->first << " namespace: " << d->nameSpace );
-      if( it->first == n
-          or ( it->first.nameSpace() == d->nameSpace and it->first.name() == n.name() ) )
-      {
-        return it->second;
-      }
-    }
-  }
-  return 0;
-}
-
-std::list<VariableNG*> ParserBase::functionVariables() const
-{
-  return d->functionVariables;
-}
-
-void ParserBase::declareVariable( const ScopedName& _scopedName, VariableNG* _variable)
-{
-  d->contextes.begin()->variables[_scopedName] = _variable;
-  d->functionVariables.push_back( _variable );
+  return token.isNativeType() or ( token.type == Token::IDENTIFIER and d->compiler->typesManager()->d->isKnownType( token.string ) );
 }
 
 std::list<int> ParserBase::expressionsListToIntegersList( const std::list< AST::Expression* >& list )
@@ -1393,6 +1624,7 @@ std::list<int> ParserBase::expressionsListToIntegersList( const std::list< AST::
 void ParserBase::setNameSpace( const String& _nameSpace )
 {
   d->nameSpace = _nameSpace;
+  d->variablesManager.setNameSpace( _nameSpace );
 }
 
 const String& ParserBase::nameSpace() const
@@ -1400,7 +1632,47 @@ const String& ParserBase::nameSpace() const
   return d->nameSpace;
 }
 
-TypeManager* ParserBase::typeManager()
+TypesManager* ParserBase::typesManager()
+{
+  return d->compiler->typesManager();
+}
+
+VariablesManager* ParserBase::variablesManager()
+{
+  return &d->variablesManager;
+}
+
+int ParserBase::vectorMemberNameToIdx( const GTLCore::String& _name )
+{
+  GTL_DEBUG(_name);
+  if( _name == "x" or _name == "r" ) {
+    return 0;
+  } else if( _name == "y" or _name == "g"  ) {
+    return 1;
+  } else if( _name == "z" or _name == "b"  ) {
+    return 2;
+  } else if( _name == "w" or _name == "a"  ) {
+    return 3;
+  }
+  GTL_DEBUG("unexpected");
+  reportUnexpected( d->currentToken );
+  return -1;
+}
+
+int ParserBase::structMemberNameToIdx( const GTLCore::Type* _type, const GTLCore::String& _name )
+{
+  int index = _type->d->memberToIndex( _name );
+  if( index == -1 )
+  {
+    reportError("Unknown member: '" + _name + "' for structure " + _type->structName(), d->currentToken );
+  }
+  return index;
+}
+
+void ParserBase::startParsingFunction( const GTLCore::String& _name )
+{
+}
+
+void ParserBase::endParsingFunction( const GTLCore::String& _name )
 {
-  return d->compiler->typeManager();
 }
