@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,10 +33,16 @@ namespace GTLCore {
       ~Region();
       int x() const;
       int y() const;
-      int width() const;
-      void setWidth( int _width );
       int height() const;
+      int width() const;
+      int left() const;
+      int right() const;
+      int bottom() const;
+      int top() const;
+      void setWidth( int _width );
       void setHeight( int _height );
+      GTLCore::Region united( const Region& _region ) const;
+      GTLCore::Region& operator+=( const Region& _region );
       bool operator!=( const Region& _region ) const;
     private:
       int m_x, m_y, m_width, m_height;
