@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,15 +20,21 @@
 #ifndef _GTLCORE_VERSION_H_
 #define _GTLCORE_VERSION_H_
 
+#include <GTLCore/String.h>
+
+#define GTL_CORE_VERSION_MAJOR 0
+#define GTL_CORE_VERSION_MINOR 9
+#define GTL_CORE_VERSION_REVISION 9
+
 namespace GTLCore {
-  const char* LibraryShortName = "OpenGTL Core";
-  const char* LibraryName = "Open Graphics Transformation Languages Core";
-  const char* LibraryCopyright = "Copyright (c) 2007-2008 Cyrille Berger (cberger@cberger.net)";
-  const char* LibraryLicence = "GNU Lesser General Public License Version 2";
-  const char* LibraryVersionString = "0.9.5";
-  int LibraryVersionMajor = 0;
-  int LibraryVersionMinor = 9;
-  int LibraryVersionRevision = 5;
+  String LibraryShortName();
+  String LibraryName();
+  String LibraryCopyright();
+  String LibraryLicence();
+  String LibraryVersionString();;
+  int LibraryVersionMajor();
+  int LibraryVersionMinor();
+  int LibraryVersionRevision();
 }
 
 #endif
