@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +24,7 @@
 #include <llvm/Constants.h>
 #include <llvm/Function.h>
 #include <llvm/Instructions.h>
+#include <llvm/GlobalVariable.h>
 
 // GTLCore
 #include "CodeGenerator_p.h"
@@ -39,6 +40,7 @@
 #include "Utils_p.h"
 
 #include "AST/Expression.h"
+#include "wrappers/ArrayWrap.h"
 
 using namespace GTLCore;
 
@@ -106,7 +108,7 @@ llvm::Value* PrimitiveVisitor::pointerToIndex(GenerationContext& _generationCont
 
 ExpressionResult PrimitiveVisitor::get( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType) const
 {
-  return ExpressionResult( new llvm::LoadInst( _pointer, "", _currentBlock), _pointerType);
+  return ExpressionResult( new llvm::LoadInst( _pointer, "PrimitiveVisitor::get", _currentBlock), _pointerType);
 }
 
 llvm::BasicBlock* PrimitiveVisitor::set( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _value, const Type* _valueType, bool _allocatedInMemory ) const
@@ -115,7 +117,7 @@ llvm::BasicBlock* PrimitiveVisitor::set( GenerationContext& _generationContext,
   GTL_DEBUG( *_pointer );
   new llvm::StoreInst(
           _generationContext.codeGenerator()->convertValueTo( _currentBlock, _value, _valueType, _pointerType ),
-          _pointer, "", _currentBlock);
+          _pointer, "PrimitiveVisitor::set", _currentBlock);
   return _currentBlock;
 }
 
@@ -125,11 +127,21 @@ llvm::BasicBlock* PrimitiveVisitor::initialise(GenerationContext& _generationCon
   return _currentBlock;
 }
 
-llvm::BasicBlock* PrimitiveVisitor::cleanUp( GenerationContext& , llvm::BasicBlock* _currentBlock, llvm::Value* , const Type* , llvm::Value* , bool ) const
+llvm::BasicBlock* PrimitiveVisitor::cleanUp( GenerationContext& , llvm::BasicBlock* _currentBlock, llvm::Value* , const Type* , llvm::Value* , bool, bool /*_ignoreCount*/, bool /*_deletePointer*/ ) const
 {
   return _currentBlock;
 }
 
+llvm::BasicBlock* PrimitiveVisitor::mark( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* increment ) const
+{
+  return _currentBlock;
+}
+
+llvm::Constant* PrimitiveVisitor::createStaticVariable( llvm::Module*, const Type* type, const std::list< int>& /*_sizes*/ ) const
+{
+  return CodeGenerator::convertConstantTo( CodeGenerator::integerToConstant( 0), Type::Integer32, type );
+}
+
 //--------- ArrayVisitor ---------///
 
 ArrayVisitor::ArrayVisitor() : Visitor(  )
@@ -165,7 +177,7 @@ llvm::BasicBlock* ArrayVisitor::set( GenerationContext& _generationContext, llvm
       std::list<llvm::Value*> sizes;
       sizes.push_back( getSize( _generationContext, ifContent, _value ) );
       
-      llvm::BasicBlock* endIfContent = cleanUp(_generationContext, ifContent, _pointer, _pointerType, 0, _allocatedInMemory );
+      llvm::BasicBlock* endIfContent = cleanUp(_generationContext, ifContent, _pointer, _pointerType, 0, _allocatedInMemory, true, false );
       
       endIfContent = initialise( _generationContext, endIfContent, _pointer, _pointerType, sizes, _allocatedInMemory );
       
@@ -176,7 +188,7 @@ llvm::BasicBlock* ArrayVisitor::set( GenerationContext& _generationContext, llvm
     
     }
     //   int i = 0;
-    VariableNG* index = new VariableNG( Type::Integer32, false);
+    VariableNG* index = new VariableNG( Type::Integer32, false, false);
     index->initialise( _generationContext, _currentBlock, ExpressionResult( _generationContext.codeGenerator()->integerToConstant(0), Type::Integer32), std::list<llvm::Value*>());
     
     // Construct the body of the for loop
@@ -214,25 +226,26 @@ llvm::BasicBlock* ArrayVisitor::setSize(GenerationContext& _generationContext, l
   GTL_DEBUG( *_size );
   std::vector<llvm::Value*> indexes;
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the structure of the array
-  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the size of the array
+  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, ArrayWrap::POS_SIZE)); // Access the size of the array
   { // Init the size
-    llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
+    GTL_DEBUG( *_pointer );
+    llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "ArrayVisitor::setSize", _currentBlock);
     new llvm::StoreInst(_generationContext.codeGenerator()->convertValueTo( _currentBlock, _size, Type::Integer32, Type::Integer32 ), ptr, "", _currentBlock);
   }
   // Allocate the Array
-  indexes[1] = llvm::ConstantInt::get(llvm::Type::Int32Ty, 1);
+  indexes[1] = llvm::ConstantInt::get(llvm::Type::Int32Ty, ArrayWrap::POS_DATA);
   {
-    llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
+    llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "ArrayVisitor::setSize", _currentBlock);
     llvm::Value* array = 0;
     if( _allocatedInMemory )
     {
-      array = new llvm::MallocInst(
-                    _pointerType->embeddedType()->d->type(), _size, "", _currentBlock);
+      array = CodeGenerator::allocateMemory( _generationContext,
+                    _pointerType->embeddedType()->d->type(), _size, _currentBlock);
     } else {
       array = new llvm::AllocaInst(
-                    _pointerType->embeddedType()->d->type(), _size, "", _currentBlock);
+                    _pointerType->embeddedType()->d->type(), _size, "ArrayVisitor::setSize", _currentBlock);
     }
-    new llvm::StoreInst( array, ptr, "", _currentBlock);
+    new llvm::StoreInst( array, ptr, "ArrayVisitor::setSize", _currentBlock);
   }
   return _currentBlock;
 }
@@ -241,15 +254,16 @@ llvm::Value* ArrayVisitor::getSize(GenerationContext& _generationContext, llvm::
 {
   std::vector<llvm::Value*> indexes;
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the structure of the array
-  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the size of the array
+  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, ArrayWrap::POS_SIZE)); // Access the size of the array
   llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", currentBlock);
-  return new llvm::LoadInst( ptr, "", currentBlock);
+  return new llvm::LoadInst( ptr, "ArrayVisitor::getSize", currentBlock);
 }
 
 llvm::BasicBlock* ArrayVisitor::initialise(GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, const std::list< llvm::Value*>& _sizes, bool _allocatedInMemory) const
 {
   GTL_ASSERT( _pointerType->dataType() == Type::ARRAY );
   GTL_DEBUG( _sizes.empty() );
+  CodeGenerator::setCountFieldOf( _currentBlock, _pointer, CodeGenerator::integerToConstant( 1 ) );
   if( not _sizes.empty())
   {
     llvm::Value* currentSize = _sizes.front();
@@ -257,7 +271,7 @@ llvm::BasicBlock* ArrayVisitor::initialise(GenerationContext& _generationContext
     std::list< llvm::Value*> sizeAfter = _sizes;
     sizeAfter.pop_front();
     //   int i = 0;
-    VariableNG* index = new VariableNG( Type::Integer32, false);
+    VariableNG* index = new VariableNG( Type::Integer32, false, false);
     index->initialise( _generationContext, _currentBlock, ExpressionResult(_generationContext.codeGenerator()->integerToConstant(0), Type::Integer32), std::list<llvm::Value*>());
     
     // Construct the body of the for loop
@@ -287,19 +301,41 @@ llvm::BasicBlock* ArrayVisitor::initialise(GenerationContext& _generationContext
   } else {
     std::vector<llvm::Value*> indexes;
     indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the structure of the array
-    indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the size of the array
+    indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, ArrayWrap::POS_SIZE)); // Access the size of the array
     // Init the size
-    llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
-    new llvm::StoreInst( GTLCore::CodeGenerator::integerToConstant( 0 ), ptr, "", _currentBlock);
+    llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "ArrayVisitor::initialise", _currentBlock);
+    new llvm::StoreInst( GTLCore::CodeGenerator::integerToConstant( 0 ), ptr, "ArrayVisitor::initialise", _currentBlock);
   }
   return _currentBlock;
 }
 
-llvm::BasicBlock* ArrayVisitor::cleanUp( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _donttouch, bool _allocatedInMemory ) const
+llvm::BasicBlock* ArrayVisitor::cleanUp( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _donttouch, bool _allocatedInMemory, bool _ignoreCount, bool _deletePointer ) const
 {
+  // TODO use _donttouch in the array as well
     //   int i = 0;
-    VariableNG* index = new VariableNG( Type::Integer32, false);
-    index->initialise( _generationContext, _currentBlock, ExpressionResult( _generationContext.codeGenerator()->integerToConstant(0), Type::Integer32), std::list<llvm::Value*>());
+#ifndef NDEBUG
+  if( _pointer )
+  {
+    GTL_DEBUG( _pointer << " " << *_pointer );
+  }
+  GTL_DEBUG( _donttouch );
+  if( _donttouch )
+  {
+    GTL_DEBUG( _donttouch  << " " << *_donttouch );
+  }
+#endif
+  if( _pointer != _donttouch )
+  {
+    llvm::Value* arraySize = getSize( _generationContext, _currentBlock, _pointer);
+    llvm::Value* test1 = CodeGenerator::createStrictInferiorExpression( _currentBlock, CodeGenerator::getCountFieldOf( _currentBlock, _pointer), Type::Integer32, CodeGenerator::integerToConstant( 1 ), Type::Integer32 );
+    llvm::Value* test2 = CodeGenerator::createDifferentExpression( _currentBlock, arraySize, Type::Integer32, CodeGenerator::integerToConstant( 0 ), Type::Integer32 );
+    llvm::Value* testIf = CodeGenerator::createAndExpression( _currentBlock, test1, Type::Boolean, test2, Type::Boolean );
+    llvm::BasicBlock* firstIfBlock = llvm::BasicBlock::Create("firstIfBlock");
+    _generationContext.llvmFunction()->getBasicBlockList().push_back( firstIfBlock);
+    
+    
+    VariableNG* index = new VariableNG( Type::Integer32, false, false);
+    index->initialise( _generationContext, firstIfBlock, ExpressionResult( _generationContext.codeGenerator()->integerToConstant(0), Type::Integer32), std::list<llvm::Value*>());
     
     // Construct the body of the for loop
     llvm::BasicBlock* bodyBlock = llvm::BasicBlock::Create("bodyBlock");
@@ -310,24 +346,106 @@ llvm::BasicBlock* ArrayVisitor::cleanUp( GenerationContext& _generationContext,
           _generationContext,
           bodyBlock, 
           _generationContext.codeGenerator()->accessArrayValue( bodyBlock, _pointer, index->get( _generationContext, bodyBlock  ) ),
-          _pointerType->embeddedType(), _donttouch, _allocatedInMemory );
-    std::vector<llvm::Value*> indexes;
-    indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the structure of the array
-    indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 1)); // Access the size of the array
-    // Init the size
-    llvm::Value* ptrToData = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", endBodyBlock);
-    new llvm::FreeInst( new llvm::LoadInst( ptrToData, "", endBodyBlock ), endBodyBlock );
+          _pointerType->embeddedType(), _donttouch, _allocatedInMemory, _ignoreCount, false );
     // Create the for statement
     llvm::BasicBlock*  afterBlock = CodeGenerator::createIterationForStatement(
                     _generationContext,
-                    _currentBlock,
+                    firstIfBlock,
                     index,
-                    getSize( _generationContext, _currentBlock, _pointer),
+                    arraySize,
                     Type::Integer32,
                     bodyBlock,
                     endBodyBlock );
+    if( _allocatedInMemory )
+    {
+      std::vector<llvm::Value*> indexes;
+      indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0)); // Access the structure of the array
+      indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, ArrayWrap::POS_DATA )); // Access the data of the array
+      // Free the array
+      llvm::Value* ptrToData = llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "ArrayVisitor::cleanUp", afterBlock);
+      CodeGenerator::freeMemory(_generationContext, new llvm::LoadInst( ptrToData, "ArrayVisitor::cleanUp", afterBlock ), afterBlock );
+      if( _deletePointer )
+      {
+        CodeGenerator::freeMemory( _generationContext, _pointer, afterBlock );
+      }
+    }
+    
+    llvm::BasicBlock* afterIfBlock = llvm::BasicBlock::Create("afterIfBlock");
+    _generationContext.llvmFunction()->getBasicBlockList().push_back( afterIfBlock);
+    
+    CodeGenerator::createIfStatement( _currentBlock, testIf, Type::Boolean, firstIfBlock, afterBlock, afterIfBlock );
+    
     delete index;
-    return afterBlock;
+    return afterIfBlock;
+  } else {
+    return _currentBlock;
+  }
+}
+
+llvm::BasicBlock* ArrayVisitor::mark( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _increment ) const
+{
+  CodeGenerator::incrementCountFieldOf( _currentBlock, _pointer, _increment );
+    
+  //   int i = 0;
+  VariableNG* index = new VariableNG( Type::Integer32, false, false);
+  index->initialise( _generationContext, _currentBlock, ExpressionResult( _generationContext.codeGenerator()->integerToConstant(0), Type::Integer32), std::list<llvm::Value*>());
+    
+  // Construct the body of the for loop
+  llvm::BasicBlock* bodyBlock = llvm::BasicBlock::Create("bodyBlock");
+  _generationContext.llvmFunction()->getBasicBlockList().push_back( bodyBlock);
+  GTL_DEBUG( " value = " << *_pointer << " type = " << *_pointer->getType() << " " << *_pointerType->embeddedType() << " " << *_pointerType );
+  const Visitor* visitor = Visitor::getVisitorFor( _pointerType->embeddedType() );
+  llvm::BasicBlock* endBodyBlock = visitor->mark( 
+        _generationContext,
+        bodyBlock, 
+        _generationContext.codeGenerator()->accessArrayValue( bodyBlock, _pointer, index->get( _generationContext, bodyBlock  ) ),
+        _pointerType->embeddedType(), _increment );
+  
+  llvm::BasicBlock*  afterBlock = CodeGenerator::createIterationForStatement(
+                  _generationContext,
+                  _currentBlock,
+                  index,
+                  getSize( _generationContext, _currentBlock, _pointer),
+                  Type::Integer32,
+                  bodyBlock,
+                  endBodyBlock );
+  delete index;
+  return afterBlock;
+}
+
+llvm::Constant* ArrayVisitor::createStaticVariable( llvm::Module* _module, const Type* type, const std::list< int>& _sizes ) const
+{
+  // Init size
+  int currentSize = _sizes.front();
+  std::list< int> sizeAfter = _sizes;
+  sizeAfter.pop_front();
+ 
+  const GTLCore::Type* embedded = type->embeddedType();
+  //
+  std::vector<llvm::Constant*> initialisers;
+  // Initialize the count
+  initialisers.push_back( CodeGenerator::integerToConstant( 1 ) );
+  // Initialize the current size
+  initialisers.push_back( CodeGenerator::integerToConstant( currentSize ) );
+  // Initialize the data
+  const Visitor* visitor = Visitor::getVisitorFor( embedded );
+  std::vector<llvm::Constant*> arrayInitialisers;
+  for( int i = 0; i < currentSize; ++i)
+  {
+    arrayInitialisers.push_back( visitor->createStaticVariable( _module, embedded, sizeAfter ) );
+  }
+  // Initialize the pointer to the data
+  llvm::ArrayType* arrType = llvm::ArrayType::get(embedded->d->type(), currentSize);
+  llvm::Constant* arrayData = new llvm::GlobalVariable( arrType, false, llvm::GlobalValue::InternalLinkage,  llvm::ConstantArray::get(arrType ,arrayInitialisers), "ArrayVisitor::createStaticVariable", _module );
+  // Get the pointer to the data
+  
+  std::vector<llvm::Constant*> indices;
+  indices.push_back( CodeGenerator::integerToConstant( 0 ) );
+  indices.push_back( CodeGenerator::integerToConstant( 0 ) );
+  initialisers.push_back( llvm::ConstantExpr::getGetElementPtr(arrayData, &indices[0], indices.size() ) );
+  
+  GTL_DEBUG(* initialisers[2] << " " <<  *type->d->type());
+  return llvm::ConstantStruct::get( dynamic_cast<const llvm::StructType*>(type->d->type()), initialisers );
 }
 
 //--------- VectorVisitor ---------///
@@ -348,12 +466,12 @@ llvm::Value* VectorVisitor::pointerToIndex(GenerationContext& _generationContext
   
   return llvm::GetElementPtrInst::Create( 
               CodeGenerator::convertPointerTo( _currentBlock, _pointer, _pointerType->embeddedType()->d->type() ),
-              _index, "", _currentBlock);
+              _index, "VectorVisitor::pointerToIndex", _currentBlock);
 }
 
 ExpressionResult VectorVisitor::get( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType) const
 {
-  return ExpressionResult(new llvm::LoadInst( _pointer, "", _currentBlock), _pointerType );
+  return ExpressionResult(new llvm::LoadInst( _pointer, "VectorVisitor::get", _currentBlock), _pointerType );
 }
 
 llvm::BasicBlock* VectorVisitor::set(GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _value, const Type* _valueType, bool _allocatedInMemory) const
@@ -362,7 +480,7 @@ llvm::BasicBlock* VectorVisitor::set(GenerationContext& _generationContext, llvm
   GTL_DEBUG( "value = " << *_value << " type = " << *_valueType );
   new llvm::StoreInst(
           _generationContext.codeGenerator()->convertValueTo( _currentBlock, _value, _valueType, _pointerType ),
-          _pointer, "", _currentBlock);
+          _pointer, "VectorVisitor::set", _currentBlock);
   return _currentBlock;
 }
 
@@ -372,12 +490,28 @@ llvm::BasicBlock* VectorVisitor::initialise( GenerationContext& _generationConte
   return _currentBlock;
 }
 
-llvm::BasicBlock* VectorVisitor::cleanUp( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _donttouch, bool _allocatedInMemory ) const
+llvm::BasicBlock* VectorVisitor::cleanUp( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _donttouch, bool _allocatedInMemory, bool /*_ignoreCount*/, bool /*_deletePointer*/ ) const
 {
   // Don't do nothing
   return _currentBlock;
 }
 
+llvm::BasicBlock* VectorVisitor::mark( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* increment ) const
+{
+  return _currentBlock;
+}
+
+llvm::Constant* VectorVisitor::createStaticVariable( llvm::Module* _module, const Type* _type, const std::list< int>& /*_sizes*/ ) const
+{
+  GTL_ASSERT( _type->dataType() == Type::VECTOR );
+  std::vector< llvm::Constant* > constants;
+  const Visitor* visitor = Visitor::getVisitorFor( _type->embeddedType() );
+  for( std::size_t i  = 0; i < _type->vectorSize(); ++i)
+  {
+    constants.push_back( visitor->createStaticVariable( _module, _type->embeddedType(), std::list<int>() ) );
+  }
+  return llvm::ConstantVector::get( dynamic_cast<const llvm::VectorType*>(_type->d->type() ), constants);
+}
 
 //--------- StructureVisitor ---------///
 
@@ -400,8 +534,8 @@ llvm::Value* StructureVisitor::pointerToValue( GenerationContext& /*_generationC
 {
   std::vector<llvm::Value*> indexes;
   indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, 0));
-  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, _index));
-  return llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock);
+  indexes.push_back( llvm::ConstantInt::get(llvm::Type::Int32Ty, _index + STRUCT_FIRST_ELEMENT ));
+  return llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "StructureVisitor::pointerToValue", _currentBlock);
 }
 
 llvm::Value* StructureVisitor::pointerToIndex(GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _type, llvm::Value* _index) const
@@ -417,14 +551,14 @@ ExpressionResult StructureVisitor::get( GenerationContext& _generationContext, l
 llvm::BasicBlock* StructureVisitor::set( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _value, const Type* _valueType, bool _allocatedInMemory ) const
 {
   GTL_ASSERT( _pointerType->dataType() == Type::STRUCTURE );
-  for(uint i = 0; i < _pointerType->structDataMembers()->size(); ++i)
+  for(uint i = 0; i < _pointerType->countStructDataMembers(); ++i)
   {
-    const Type* type = (*_pointerType->structDataMembers())[i].type();
+    const Type* type = _pointerType->structDataMember(i).type();
     llvm::Value* nptrToOwnMember = pointerToValue(_generationContext, _currentBlock, _pointer, i);
     llvm::Value* nptrToValueMember = pointerToValue( _generationContext, _currentBlock, _value, i);
     const Visitor* visitor = Visitor::getVisitorFor( type );
     llvm::Value* memberValue = visitor->get( _generationContext, _currentBlock, nptrToValueMember, type ).value();
-    visitor->set( _generationContext, _currentBlock, nptrToOwnMember, type, memberValue, type, _allocatedInMemory );
+    _currentBlock = visitor->set( _generationContext, _currentBlock, nptrToOwnMember, type, memberValue, type, _allocatedInMemory );
   }
   return _currentBlock;
 }
@@ -432,36 +566,88 @@ llvm::BasicBlock* StructureVisitor::set( GenerationContext& _generationContext,
 llvm::BasicBlock* StructureVisitor::initialise(GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, const std::list< llvm::Value*>& _sizes, bool _allocatedInMemory) const
 {
   GTL_ASSERT( _pointerType->dataType() == Type::STRUCTURE );
-  const std::vector<Type::StructDataMember>* sm = _pointerType->structDataMembers();
   
-  for( uint i = 0; i < sm->size(); ++i)
+  for( uint i = 0; i < _pointerType->countStructDataMembers(); ++i)
   {
     std::list< llvm::Value* > sizes;
-    for(std::list<int>::const_iterator it = (*sm)[i].initialSizes().begin();
-        it != (*sm)[i].initialSizes().end(); ++it)
+    const Type::StructDataMember& sdm = _pointerType->structDataMember(i);
+    for(std::list<int>::const_iterator it = sdm.initialSizes().begin();
+        it != sdm.initialSizes().end(); ++it)
     {
       sizes.push_back( _generationContext.codeGenerator()->integerToConstant( *it ) );
     }
-    const Type* type = (*sm)[i].type();
+    const Type* type = sdm.type();
     const Visitor* visitor = Visitor::getVisitorFor( type );
     _currentBlock = visitor->initialise( _generationContext, _currentBlock,
                             pointerToValue( _generationContext, _currentBlock, _pointer, i ),
                             type, sizes, _allocatedInMemory );
   }
+  CodeGenerator::setCountFieldOf( _currentBlock, _pointer, CodeGenerator::integerToConstant( 1 ) );
   return _currentBlock;
 }
 
-llvm::BasicBlock* StructureVisitor::cleanUp( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _donttouch, bool _allocatedInMemory ) const
+llvm::BasicBlock* StructureVisitor::cleanUp( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _donttouch, bool _allocatedInMemory, bool _ignoreCount, bool _deletePointer ) const
 {
-  if( _pointer == _donttouch ) return _currentBlock;
-  GTL_ASSERT( _pointerType->dataType() == Type::STRUCTURE );
-  const std::vector<Type::StructDataMember>* sm = _pointerType->structDataMembers();
-  
-  for( uint i = 0; i < sm->size(); ++i)
+  // TODO use ignoreCount
+#ifndef NDEBUG
+  if( _pointer )
   {
-    const Type* type = (*sm)[i].type();
+    GTL_DEBUG( _pointer << " " << *_pointer );
+  }
+  if( _donttouch )
+  {
+    GTL_DEBUG( _donttouch  << " " << *_donttouch );
+  }
+#endif
+  if( _pointer == _donttouch ) return _currentBlock;
+  
+  llvm::Value* test = CodeGenerator::createStrictInferiorExpression( _currentBlock, CodeGenerator::getCountFieldOf( _currentBlock, _pointer), Type::Integer32, CodeGenerator::integerToConstant( 1 ), Type::Integer32 );
+  llvm::BasicBlock* firstIfBlock = llvm::BasicBlock::Create("firstIfBlockStructureVisitorCleanUp");
+  _generationContext.llvmFunction()->getBasicBlockList().push_back( firstIfBlock);
+  llvm::BasicBlock* lastIfBlock = firstIfBlock;
+  
+  GTL_ASSERT( _pointerType->dataType() == Type::STRUCTURE );
+  
+  for( uint i = 0; i < _pointerType->countStructDataMembers(); ++i)
+  {
+    const Type* type = _pointerType->structDataMember(i).type();
     const Visitor* visitor = Visitor::getVisitorFor( type );
-    _currentBlock = visitor->cleanUp( _generationContext, _currentBlock, pointerToValue( _generationContext, _currentBlock, _pointer, i ), type, _donttouch, _allocatedInMemory );
+    lastIfBlock = visitor->cleanUp( _generationContext, lastIfBlock, pointerToValue( _generationContext, lastIfBlock, _pointer, i ), type, _donttouch, _allocatedInMemory, _ignoreCount, false );
+  }
+  
+  llvm::BasicBlock* afterIfBlock = llvm::BasicBlock::Create("afterIfBlockStructureVisitorCleanUp");
+  _generationContext.llvmFunction()->getBasicBlockList().push_back( afterIfBlock);
+  if( _allocatedInMemory and _deletePointer )
+  {
+    CodeGenerator::freeMemory( _generationContext, _pointer, lastIfBlock );
+  }
+  CodeGenerator::createIfStatement( _currentBlock, test, Type::Boolean, firstIfBlock, lastIfBlock, afterIfBlock );
+  
+  return afterIfBlock;
+}
+
+llvm::BasicBlock* StructureVisitor::mark( GenerationContext& _generationContext, llvm::BasicBlock* _currentBlock, llvm::Value* _pointer, const Type* _pointerType, llvm::Value* _increment ) const
+{
+  CodeGenerator::incrementCountFieldOf( _currentBlock, _pointer, _increment );
+  for( uint i = 0; i < _pointerType->countStructDataMembers(); ++i)
+  {
+    const Type* type = _pointerType->structDataMember(i).type();
+    const Visitor* visitor = Visitor::getVisitorFor( type );
+    _currentBlock = visitor->mark( _generationContext, _currentBlock, pointerToValue( _generationContext, _currentBlock, _pointer, i ), type, _increment );
   }
   return _currentBlock;
 }
+
+llvm::Constant* StructureVisitor::createStaticVariable( llvm::Module* _module, const Type* _type, const std::list< int>& /*_sizes */) const
+{
+  GTL_ASSERT( _type->dataType() == Type::STRUCTURE );
+  std::vector< llvm::Constant* > initialisers;
+  initialisers.push_back( CodeGenerator::integerToConstant( 1 ) );
+  for( uint i = 0; i < _type->countStructDataMembers(); ++i)
+  {
+    const Type* type = _type->structDataMember(i).type();
+    const Visitor* visitor = Visitor::getVisitorFor( type );
+    initialisers.push_back( visitor->createStaticVariable( _module, type, _type->structDataMember(i).initialSizes() ) );
+  }
+  return llvm::ConstantStruct::get( dynamic_cast<const llvm::StructType*>(_type->d->type()), initialisers );
+}
