@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,14 +24,41 @@
 
 namespace GTLImageIO {
   class ImageDC;
+  /**
+   * This is a registry of all \ref ImageDC coder/decoder that are available.
+   * 
+   * @ingroup GTLImageIO
+   */
   class ImageDCRegistry {
       ImageDCRegistry();
       virtual ~ImageDCRegistry();
       void initialise();
     public:
+      /**
+       * @return the registry instance.
+       */
       static ImageDCRegistry* instance();
+      /**
+       * Filter for use in dialogs for the list of all image types that can be
+       * decoded.
+       */
+      const GTLCore::String& decodableFilter() const;
+      /**
+       * Filter for use in dialogs for the list of all image types that can be
+       * encoded.
+       */
+      const GTLCore::String& encodableFilter() const;
+      /**
+       * Add a \ref ImageDC to the registry.
+       */
       void registerDC( ImageDC* _imageDC );
+      /**
+       * Find the decoder for the given image.
+       */
       const ImageDC* decoder( const GTLCore::String& _fileName ) const;
+      /**
+       * Find the encoder for the given image.
+       */
       const ImageDC* encoder( const GTLCore::String& _fileName ) const;
     private:
       struct Private;
