@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +29,6 @@
 #include <llvm/Instructions.h>
 #include <llvm/Module.h>
 #include <llvm/Value.h>
-#include <llvm/ParameterAttributes.h>
 
 #include "GTLCore/CodeGenerator_p.h"
 #include "GTLCore/ExpressionResult_p.h"
@@ -40,7 +39,7 @@
 #include "GTLCore/Parameter.h"
 #include "GTLCore/PixelDescription.h"
 #include "GTLCore/Type.h"
-#include "GTLCore/TypeManager_p.h"
+#include "GTLCore/TypesManager_p.h"
 #include "GTLCore/Value.h"
 #include "GTLCore/VariableNG_p.h"
 
@@ -86,7 +85,7 @@ llvm::Value* CodeGenerator::accessPixelDataPtr( GTLCore::GenerationContext& _gc,
   SHIVA_DEBUG( *_pixel );
   std::vector<llvm::Value*> indexes;
   indexes.push_back( _gc.codeGenerator()->integerToConstant(0));
-  indexes.push_back( _gc.codeGenerator()->integerToConstant(PixelWrap::INDEX_DATA));
+  indexes.push_back( _gc.codeGenerator()->integerToConstant(PixelWrap::POS_DATA ));
   return llvm::GetElementPtrInst::Create( _pixel, indexes.begin(), indexes.end(), "", _currentBlock );
 }
 
@@ -154,22 +153,23 @@ void CodeGenerator::setPixelCoordinates( GTLCore::GenerationContext& _gc, llvm::
 {
   SHIVA_DEBUG("setPixelCoordinates");
   // Create the coord vector
-  llvm::Value* result = llvm::ConstantVector::get( static_cast<const llvm::VectorType*>( GTLCore::TypeManager::getVector( GTLCore::Type::Float, 2)->d->type()), std::vector<llvm::Constant*>() );
+  llvm::Value* result = GTLCore::CodeGenerator::createVector( 2, GTLCore::CodeGenerator::floatToConstant( 0.0 ),  GTLCore::Type::Float );
   SHIVA_DEBUG(*result << " " << *_iVal << " " << *_jVal);
   result = llvm::InsertElementInst::Create( result, GTLCore::CodeGenerator::convertValueTo( _currentBlock, _iVal, _iValType, GTLCore::Type::Float), (unsigned int)0, "", _currentBlock );
   result = llvm::InsertElementInst::Create( result, GTLCore::CodeGenerator::convertValueTo( _currentBlock, _jVal, _iValType, GTLCore::Type::Float), 1, "", _currentBlock );
   
   std::vector<llvm::Value*> indexes;
   indexes.push_back( _gc.codeGenerator()->integerToConstant(0));
-  indexes.push_back( _gc.codeGenerator()->integerToConstant(PixelWrap::INDEX_COORD));
-  new llvm::StoreInst( result,
-        llvm::GetElementPtrInst::Create( _pixel, indexes.begin(), indexes.end(), "", _currentBlock ),
-        "", _currentBlock );
+  indexes.push_back( _gc.codeGenerator()->integerToConstant(PixelWrap::POS_COORD ));
+  llvm::Value* ptr = llvm::GetElementPtrInst::Create( _pixel, indexes.begin(), indexes.end(), "", _currentBlock );
+  SHIVA_DEBUG( *_pixel << " " << *ptr << " " << *result);
+  new llvm::StoreInst( result, ptr, "", _currentBlock );
 }
 
-llvm::Function* CodeGenerator::generateEvaluatePixeles( const std::vector<const GTLCore::Type*>& _inputTypes, const GTLCore::Type* _outputType, Kernel* _kernel, GTLCore::ModuleData* _moduleData, int _channels)
+llvm::Function* CodeGenerator::generateEvaluatePixeles( Kernel* _kernel, int _channels)
 {
-  GTLCore::CodeGenerator codeGenerator( _moduleData );
+  GTLCore::ModuleData* moduleData = _kernel->d->moduleData();
+  GTLCore::CodeGenerator codeGenerator( moduleData );
   
   // Pseudo-code explaining step-by-step what an evaluatePixeles function does :
   //
@@ -201,16 +201,14 @@ llvm::Function* CodeGenerator::generateEvaluatePixeles( const std::vector<const
   params.push_back( llvm::Type::Int32Ty );
   params.push_back( llvm::Type::Int32Ty );
   params.push_back( llvm::Type::Int32Ty );
-  SHIVA_ASSERT( _moduleData->typeManager()->getStructure( "image" ) );
-  params.push_back( 
-          llvm::PointerType::get( 
-                   llvm::PointerType::get( _moduleData->typeManager()->getStructure( "image" )->d->type(), 0 ), 0 ) );
-  params.push_back( llvm::PointerType::get( _moduleData->typeManager()->getStructure( "image" )->d->type(), 0 ) );
+  SHIVA_ASSERT( _kernel->d->outputImageType() );
+  params.push_back( llvm::PointerType::get( GTLCore::Type::Pointer->d->type(), 0 ) );
+  params.push_back( llvm::PointerType::get( _kernel->d->outputImageType()->d->type(), 0 ) );
   llvm::FunctionType* definitionType = llvm::FunctionType::get( llvm::Type::VoidTy, params, false );
   int evaluatePixelesId = ++CodeGenerator::s_evaluatePixelesId;
   llvm::Function* func = codeGenerator.createFunction( definitionType, "evaluatePixeles" + GTLCore::String::number(evaluatePixelesId));
   // Initialise a generation context
-  GTLCore::GenerationContext generationContext( &codeGenerator, func, 0, _moduleData );
+  GTLCore::GenerationContext generationContext( &codeGenerator, func, 0, moduleData );
   
   // {
     llvm::BasicBlock* initialBlock = llvm::BasicBlock::Create();
@@ -235,10 +233,10 @@ llvm::Function* CodeGenerator::generateEvaluatePixeles( const std::vector<const
     ++arg_it;
     llvm::Value* arg_result = arg_it;
   // Create the pixel
-    GTLCore::VariableNG* resultVar = new GTLCore::VariableNG( _moduleData->typeManager()->getStructure( "pixel" ), false );
+    GTLCore::VariableNG* resultVar = new GTLCore::VariableNG( _kernel->d->outputPixelType(), false, false );
   resultVar->initialise( generationContext, initialBlock, GTLCore::ExpressionResult(), std::list<llvm::Value*>());
   // Get the evaluatePixel function
-    GTLCore::Function* ePFunction = _moduleData->function( _kernel->name(), "evaluatePixel" );
+    GTLCore::Function* ePFunction = moduleData->function( _kernel->name(), "evaluatePixel" );
     SHIVA_ASSERT( ePFunction );
     unsigned int countArguments = ePFunction->parameters().size();
     
@@ -246,16 +244,16 @@ llvm::Function* CodeGenerator::generateEvaluatePixeles( const std::vector<const
     std::vector<llvm::Value*> evaluatePixel_params;
     for(unsigned int i = 0; i < countArguments - 1; ++i )
     {
-      evaluatePixel_params.push_back(
-          new llvm::LoadInst(
+      llvm::Value* imgPtr = new llvm::LoadInst(
               llvm::GetElementPtrInst::Create( arg_sources, GTLCore::CodeGenerator::integerToConstant( i ), "", initialBlock ),
-              "", initialBlock ) );
+              "", initialBlock );
+      evaluatePixel_params.push_back( GTLCore::CodeGenerator::convertPointerTo( initialBlock, imgPtr, ePFunction->parameters()[i].type()->d->type() ) );
     }
-    evaluatePixel_params.push_back( resultVar->pointer() );
+    evaluatePixel_params.push_back( resultVar->pointer(initialBlock) );
     SHIVA_ASSERT( evaluatePixel_params.size() == countArguments );
   // Construct the "conditions" of the first loop
     // int j = 0;
-    GTLCore::VariableNG* incJ = new GTLCore::VariableNG( GTLCore::Type::Integer32, false);
+    GTLCore::VariableNG* incJ = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false);
     incJ->initialise( generationContext, initialBlock,
                       GTLCore::ExpressionResult( arg_x, GTLCore::Type::Integer32),
                       std::list<llvm::Value*>());
@@ -265,7 +263,7 @@ llvm::Function* CodeGenerator::generateEvaluatePixeles( const std::vector<const
       func->getBasicBlockList().push_back( firstBlockJLoop );
       
       // int i = 0;
-      GTLCore::VariableNG* incI = new GTLCore::VariableNG( GTLCore::Type::Integer32, false);
+      GTLCore::VariableNG* incI = new GTLCore::VariableNG( GTLCore::Type::Integer32, false, false);
       incI->initialise( generationContext, firstBlockJLoop,
                         GTLCore::ExpressionResult( arg_y, GTLCore::Type::Integer32),
                         std::list<llvm::Value*>());
@@ -277,24 +275,26 @@ llvm::Function* CodeGenerator::generateEvaluatePixeles( const std::vector<const
         llvm::Value* iVal = incI->get( generationContext, firstBlockILoop );
         
         // Set the coordinates of the pixel
-        setPixelCoordinates( generationContext, firstBlockILoop, resultVar->pointer(), iVal, GTLCore::Type::Integer32, jVal, GTLCore::Type::Integer32 );
+        setPixelCoordinates( generationContext, firstBlockILoop, resultVar->pointer(firstBlockILoop), iVal, GTLCore::Type::Integer32, jVal, GTLCore::Type::Integer32 );
         
         // Call evaluatePixel
         llvm::Function* llvmEPFunction = ePFunction->d->data->function( countArguments );
         SHIVA_ASSERT( llvmEPFunction );
         SHIVA_DEBUG( evaluatePixel_params.size() );
+        GTL_COMPARE_FUNCTION_PARAMETERS( llvmEPFunction, evaluatePixel_params );
         llvm::CallInst::Create( llvmEPFunction, evaluatePixel_params.begin(), evaluatePixel_params.end(), "", firstBlockILoop );
         
         // Synchronize the output pixel with input
         // Call image_wrap_data on the result to get the pointer on destination data
-        llvm::Value* pointer = callImageWrapData( generationContext, firstBlockILoop, generationContext.module()->typeManager()->getStructure( "image" ), arg_result, iVal, jVal);
-        llvm::BasicBlock* lastBlockILoop = pixelToMem( generationContext, firstBlockILoop, resultVar->pointer(), pointer, arg_result );
+        llvm::Value* pointer = callImageWrapData( generationContext, firstBlockILoop, _kernel->d->outputImageType(), arg_result, iVal, jVal);
+        llvm::BasicBlock* lastBlockILoop = pixelToMem( generationContext, firstBlockILoop, resultVar->pointer(firstBlockILoop), pointer, arg_result );
       // }
       
       llvm::BasicBlock* lastBlockJLoop = GTLCore::CodeGenerator::createIterationForStatement( generationContext, firstBlockJLoop, incI, arg_width, GTLCore::Type::Integer32, firstBlockILoop, lastBlockILoop);
     // }
     
     llvm::BasicBlock* lastBlock = GTLCore::CodeGenerator::createIterationForStatement( generationContext, initialBlock, incJ, arg_height, GTLCore::Type::Integer32, firstBlockJLoop, lastBlockJLoop);
+//     resultVar->cleanUp( generationContext, lastBlock, 0); // FIXME don't leak result pixels
     llvm::ReturnInst::Create(lastBlock);
   // Cleanup
   
@@ -340,7 +340,7 @@ llvm::Function* CodeGenerator::generateMemToVec( GTLCore::ModuleData* _moduleDat
   GTLCore::CodeGenerator codeGenerator( _moduleData );
 
   llvm::Function* func = codeGenerator.createFunction( 
-      Wrapper::image_wrap_mem_to_vec_float_type( _moduleData->typeManager(), _pixelDescription.channels()) ,
+      Wrapper::image_wrap_mem_to_vec_float_type( _moduleData->typesManager(), _pixelDescription.channels()) ,
       "image_wrap_memToVec" + GTLCore::String::number( ++memToVecId) );
   // Initialise a generation context
   GTLCore::GenerationContext generationContext( &codeGenerator, func, 0, _moduleData );
@@ -403,7 +403,12 @@ llvm::Function* CodeGenerator::generateMemToVec( GTLCore::ModuleData* _moduleDat
         }
         
         // result[i] = convert(_imgData + pos(i) )
-        floatVec = llvm::InsertElementInst::Create( floatVec, floatValue, i, "", currentBlock);
+        if( channelsNb == 1 )
+        { // Special case, special hack, for a pixel of one channel, the vec isn't a vec
+          floatVec = floatValue;
+        } else {
+          floatVec = llvm::InsertElementInst::Create( floatVec, floatValue, _pixelDescription.channelPositions()[i], "", currentBlock);
+        }
         GTL_ASSERT(channelType->bitsSize() % 8 == 0);
         currentPos += channelType->bitsSize() / 8;
   //      }
@@ -453,7 +458,7 @@ llvm::Function* CodeGenerator::generateVecToMem( GTLCore::ModuleData* _moduleDat
   GTLCore::CodeGenerator codeGenerator( _moduleData );
   
   llvm::Function* func = codeGenerator.createFunction( 
-      Wrapper::image_wrap_vec_float_to_mem_type( _moduleData->typeManager(), _pixelDescription.channels()) ,
+      Wrapper::image_wrap_vec_float_to_mem_type( _moduleData->typesManager(), _pixelDescription.channels()) ,
       "image_wrap_vecToMem" + GTLCore::String::number( ++vecToMemId) );
   // Initialise a generation context
   GTLCore::GenerationContext generationContext( &codeGenerator, func, 0, _moduleData );
@@ -487,7 +492,13 @@ llvm::Function* CodeGenerator::generateVecToMem( GTLCore::ModuleData* _moduleDat
       {
         const GTLCore::Type* channelType = _pixelDescription.channelTypes()[i];
         // _src[i]
-        llvm::Value* floatValue = new llvm::ExtractElementInst( floatVec, i, "", currentBlock );
+        llvm::Value* floatValue;
+        if( channelsNb == 1 )
+        {
+          floatValue = floatVec;
+        } else {
+          floatValue = new llvm::ExtractElementInst( floatVec, _pixelDescription.channelPositions()[i], "", currentBlock );
+        }
   //        convert( _src[i] );
         // Scale
         switch( channelType->dataType() )
@@ -533,14 +544,18 @@ llvm::Value* CodeGenerator::callVirtualMember( GTLCore::GenerationContext& _gc,
 {
   std::vector<llvm::Value*> indexes;
   indexes.push_back( _gc.codeGenerator()->integerToConstant(0));
-  indexes.push_back( _gc.codeGenerator()->integerToConstant(_member_index));
+  indexes.push_back( _gc.codeGenerator()->integerToConstant(_member_index + STRUCT_FIRST_ELEMENT));
+  GTL_DEBUG( "Call " << _member_index << " on " << *_pointer );
   
   llvm::Value* funcPtr = new llvm::LoadInst( llvm::GetElementPtrInst::Create( _pointer, indexes.begin(), indexes.end(), "", _currentBlock ), "" , _currentBlock);
+  GTL_DEBUG( *funcPtr );
+  GTL_COMPARE_FUNCTION_PARAMETERS( funcPtr, _arguments );
   return llvm::CallInst::Create( funcPtr, _arguments.begin(), _arguments.end(), "", _currentBlock );
 }
 
 llvm::Value* CodeGenerator::callImageWrapData( GTLCore::GenerationContext& _gc, llvm::BasicBlock* _currentBlock, const GTLCore::Type* _imageType, llvm::Value* _imageWrap, llvm::Value* _x, llvm::Value* _y )
 {
+  GTL_DEBUG( "CodeGenerator::callImageWrapData " << *_imageType << " " << *_imageWrap << " " << *_x << " " << *_y);
   GTL_ASSERT( _imageType );
   std::vector<llvm::Value*> image_wrap_data_params;
   image_wrap_data_params.push_back( _imageWrap );
@@ -548,25 +563,7 @@ llvm::Value* CodeGenerator::callImageWrapData( GTLCore::GenerationContext& _gc,
   image_wrap_data_params.push_back( _y );
   
   llvm::Function* func = Wrapper::image_wrap_dataFunction( _gc.llvmModule(), _imageType );
-#ifndef _NDEBUG_
-  {
-    const llvm::FunctionType *FTy =
-      llvm::cast<llvm::FunctionType>(llvm::cast<llvm::PointerType>(func->getType())->getElementType());
-    
-    GTL_ASSERT( image_wrap_data_params.size() == FTy->getNumParams() or
-            (FTy->isVarArg() and image_wrap_data_params.size() > FTy->getNumParams()) );
-    
-    for (unsigned i = 0; i < image_wrap_data_params.size(); ++i) {
-      if( i < FTy->getNumParams() and (FTy->getParamType(i) != image_wrap_data_params[i]->getType()) )
-      {
-        GTL_DEBUG( "Wrong parameter " << i << " : " << FTy->getParamType(i) << " => " << *FTy->getParamType(i) << " but got " << image_wrap_data_params[i] << " => " << *image_wrap_data_params[i]->getType() );
-      } else {
-        GTL_DEBUG( "Parameter " << i << " : " << FTy->getParamType(i) << " => " << *FTy->getParamType(i) << " but got " << image_wrap_data_params[i]->getType() << " => " << *image_wrap_data_params[i]->getType() );
-      }
-    }
-  }
-#endif
-
+  GTL_COMPARE_FUNCTION_PARAMETERS( func, image_wrap_data_params );
   
   return llvm::CallInst::Create(
       func, image_wrap_data_params.begin(), image_wrap_data_params.end(), "", _currentBlock ); // TODO get the real type of the output image (even if it doesn't matter much currently)
@@ -591,7 +588,7 @@ llvm::Function* CodeGenerator::generateImageSampleNearest(GTLCore::ModuleData* _
   GTLCore::CodeGenerator codeGenerator( _moduleData );
     
   llvm::Function* func = codeGenerator.createFunction(
-      Wrapper::image_wrap_sample_nearest_type( _moduleData->typeManager(), _imageType, _pixelType),
+      Wrapper::image_wrap_sample_nearest_type( _moduleData->typesManager(), _imageType, _pixelType),
       "image_wrap_sample_nearest" + GTLCore::String::number( ++imageSampleNearestId) );
   GTLCore::GenerationContext generationContext( &codeGenerator, func, 0, _moduleData );
   // Get the args.
@@ -612,7 +609,8 @@ llvm::Function* CodeGenerator::generateImageSampleNearest(GTLCore::ModuleData* _
                           GTLCore::CodeGenerator::integerToConstant( 1 ) );
     llvm::Value* y_i = GTLCore::CodeGenerator::createRound( currentBlock, y_f );
   //    pixelX* px = new pixelX;
-    llvm::Value* px_var = new llvm::MallocInst( _pixelType->d->type(), GTLCore::CodeGenerator::integerToConstant( 1 ), "", currentBlock );
+    llvm::Value* px_var = GTLCore::CodeGenerator::allocateMemory( generationContext, _pixelType->d->type(), GTLCore::CodeGenerator::integerToConstant( 1 ), currentBlock );
+    GTLCore::CodeGenerator::setCountFieldOf( currentBlock, px_var, GTLCore::CodeGenerator::integerToConstant( 0 ) );
   //    memToPixel( image, px, image_wrap_data(self, x, y ) );
     currentBlock = memToPixel( generationContext, currentBlock,
                                callImageWrapData( generationContext, currentBlock, _imageType, arg_self, x_i, y_i ),
