@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,18 +23,28 @@
 #include <GTLCore/String.h>
 #include <GTLCore/CompilerBase_p.h>
 
+#include <map>
+
 namespace GTLCore {
   class ModuleData;
+  class Value;
 }
 
 namespace OpenShiva {
-  
+  /**
+   * @internal
+   * @ingroup OpenShiva
+   */
   class Compiler : public GTLCore::CompilerBase {
     public:
-      Compiler();
+      Compiler( bool _isKernel, int _channelsNb );
       ~Compiler();
-      bool compile(const GTLCore::String& _sourceCode, const GTLCore::String& _kernelName, GTLCore::ModuleData* _moduleData, GTLCore::String& _nameSpace);
-      virtual GTLCore::TypeManager* typeManager();
+      bool compile(bool _isStdLibrary, const GTLCore::String& _sourceCode, const GTLCore::String& _kernelName, GTLCore::ModuleData* _moduleData, GTLCore::String& _nameSpace, const std::map< GTLCore::String, GTLCore::Value >& parameters );
+      bool importModule(const GTLCore::String& name);
+      virtual GTLCore::TypesManager* typesManager();
+      bool isKernel() const;
+      bool isStdLib() const;
+      const std::map< GTLCore::String, GTLCore::Value >& parameters() const;
     public:
       GTLCore::AST::Expression* standardConstant( const GTLCore::String& _name );
     private:
