@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,7 +40,11 @@ namespace OpenShiva {
       virtual GTLCore::AST::Tree* tree();
       virtual GTLCore::AST::Statement* parseStatement();
       virtual const GTLCore::Type* parseType();
+    protected: // Events
+      virtual void startParsingFunction( const GTLCore::String& _name );
+      virtual void endParsingFunction( const GTLCore::String& _name );
     private:
+      void parseDependentDeclaration();
       void parseKernelBody();
     private:
       struct Private;
