@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,6 +20,8 @@
 #ifndef _IMAGE_WRAP_P_H_
 #define _IMAGE_WRAP_P_H_
 
+#include "GTLCore/wrappers/StructWrap.h"
+
 namespace GTLCore {
   class AbstractImage;
 }
@@ -30,6 +32,7 @@ namespace GTLCore {
 // Wrapper::createImageType !                          //
 //---------------------- WARNING ----------------------//
 struct ImageWrap {
+  STRUCT_HEADER
   GTLCore::AbstractImage* image;
   void* memToVec;
   void* vecToMem;
@@ -38,6 +41,11 @@ struct ImageWrap {
     INDEX_MEM_TO_VEC = 1,
     INDEX_VEC_TO_MEM = 2
   };
+  enum ImagePoses {
+    POS_IMAGE = STRUCT_FIRST_ELEMENT + INDEX_IMAGE,
+    POS_MEM_TO_VEC = STRUCT_FIRST_ELEMENT + INDEX_MEM_TO_VEC,
+    POS_VEC_TO_MEM = STRUCT_FIRST_ELEMENT + INDEX_VEC_TO_MEM
+  };
 };
 
 extern "C" {
