@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,18 +20,25 @@
 #ifndef _PIXEL_WRAP_P_H_
 #define _PIXEL_WRAP_P_H_
 
+#include "GTLCore/wrappers/StructWrap.h"
+
 //---------------------- WARNING ----------------------//
 // Whenever the following structure is edited,         //
 // it's llvm declaration must be changed too in        //
 // Wrapper::createPixelType !                          //
 //---------------------- WARNING ----------------------//
 struct PixelWrap {
+  STRUCT_HEADER
   void* data;
   void* coord;
   enum PixelIndexes {
     INDEX_DATA = 0,
     INDEX_COORD = 1
   };
+  enum PixelPoses {
+    POS_DATA = STRUCT_FIRST_ELEMENT + INDEX_DATA,
+    POS_COORD = STRUCT_FIRST_ELEMENT + INDEX_COORD
+  };
 };
 
 #endif
