@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation;
- * version 2 of the License.
+ * either version 2, or (at your option) any later version of the License.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +24,13 @@
 using namespace GTLCore;
 
 #include "TestLexer.h"
+#include "TestMetadataLexer.h"
+#include "TestMetadataParser.h"
+#include "TestSource.h"
 
 GTLTEST_MAIN_BEGIN(TestOpenShiva)
 GTLTEST_MAIN_ADD_CASE(TestLexer)
+GTLTEST_MAIN_ADD_CASE(TestMetadataLexer )
+GTLTEST_MAIN_ADD_CASE(TestMetadataParser)
+GTLTEST_MAIN_ADD_CASE(TestSource)
 GTLTEST_MAIN_END()
